@@ -1,3 +1,17 @@
+# Changes in 5.6.0
+# ================
+#
+# - ROLLOUT SUPPORT
+#
+# ns_sendmail and its rollout support are now DEPRECATED in
+# OpenACS. Use acs_mail_lite::send instead.
+#
+# acs-mail-lite provides rollout support for acs_mail_lite::send and
+# implements ns_sendmail as a wrapper to it for backward
+# compatibility. See acs-mail-lite package parameters to set rollout
+# support.
+#
+
 ns_log notice "nsd.tcl: starting to read config file..."
 
 ###################################################################### 
@@ -17,8 +31,8 @@ set httpsport                 8443
 
 # The hostname and address should be set to actual values.
 # setting the address to 0.0.0.0 means aolserver listens on all interfaces
-set hostname                  [ns_info hostname]
-set address                   [ns_info address]
+set hostname                  localhost
+set address                   127.0.0.1
 
 # Note: If port is privileged (usually < 1024), OpenACS must be
 # started by root, and, in AOLserver 4, the run script have a 
@@ -27,7 +41,7 @@ set address                   [ns_info address]
 set server                    "service0" 
 set servername                "New OpenACS Installation - Development"
 
-set serverroot                "/var/lib/aolserver/${server}"
+set serverroot                "/var/www/${server}"
 
 #---------------------------------------------------------------------
 # which database do you want? postgres or oracle
@@ -35,7 +49,7 @@ set database              postgres
 
 set db_name               $server
 
-if { $database == "oracle" } {
+if { $database eq "oracle" } {
     set db_password           "mysitepassword"
 } else {
     set db_host               localhost
@@ -47,8 +61,12 @@ if { $database == "oracle" } {
 # if debug is false, all debugging will be turned off
 set debug false
 
-set homedir                   /usr/local/aolserver
-set bindir                    [file dirname [ns_info nsd]] 
+set homedir                   /usr/lib/aolserver4
+set bindir                    ${homedir}/bin
+
+
+set max_file_upload_mb        20
+set max_file_upload_min        5
 
 ###################################################################### 
 #
@@ -78,6 +96,7 @@ ns_section ns/parameters
     # maxkeepalive is ignored in aolserver4.x
     ns_param   maxkeepalive       0
     ns_param   logroll            on
+    ns_param   logmaxbackup       10
     ns_param   maxbackup          5
     ns_param   debug              $debug
 #    ns_param   mailhost           localhost 
@@ -96,7 +115,7 @@ ns_section ns/parameters
 ns_section ns/threads 
     ns_param   mutexmeter         true      ;# measure lock contention 
     # The per-thread stack size must be a multiple of 8k for AOLServer to run under MacOS X
-    ns_param   stacksize          [expr 128 * 8192]
+    ns_param   stacksize          [expr {128 * 8192}]
 
 # 
 # MIME types. 
@@ -105,8 +124,8 @@ ns_section ns/mimetypes
     #  Note: AOLserver already has an exhaustive list of MIME types:
     #  see: /usr/local/src/aolserver-4.{version}/aolserver/nsd/mimetypes.c
     #  but in case something is missing you can add it here. 
-    ns_param   Default            text/plain
-    ns_param   NoExtension        text/plain
+    ns_param   Default            */*
+    ns_param   NoExtension        */*
     ns_param   .pcd               image/x-photo-cd
     ns_param   .prc               application/x-pilot
     ns_param   .xls               application/vnd.ms-excel
@@ -178,30 +197,6 @@ ns_section ns/server/${server}/tcl
     ns_param   autoclose          on 
     ns_param   debug              $debug
  
-#---------------------------------------------------------------------
-#
-# Rollout email support
-#
-# These procs help manage differing email behavior on 
-# dev/staging/production.
-#
-#---------------------------------------------------------------------
-ns_section ns/server/${server}/acs/acs-rollout-support
-
-    # EmailDeliveryMode can be:
-    #   default:  Email messages are sent in the usual manner.
-    #   log:      Email messages are written to the server's error log.
-    #   redirect: Email messages are redirected to the addresses specified 
-    #             by the EmailRedirectTo parameter.  If this list is absent 
-    #             or empty, email messages are written to the server's error log.
-    #   filter:   Email messages are sent to in the usual manner if the 
-    #             recipient appears in the EmailAllow parameter, otherwise they 
-    #             are logged.
-
-#    ns_param   EmailDeliveryMode redirect
-#    ns_param   EmailRedirectTo    somenerd@yourdomain.test, othernerd@yourdomain.test
-#    ns_param   EmailAllow         somenerd@yourdomain.test,othernerd@yourdomain.test
-
 #---------------------------------------------------------------------
 #
 # WebDAV Support (optional, requires oacs-dav package to be installed
@@ -239,9 +234,31 @@ ns_section ns/server/${server}/module/nssock
     ns_param   port               $httpport
 # setting maxinput higher than practical may leave the server vulnerable to resource DoS attacks
 # see http://www.panoptic.com/wiki/aolserver/166
-    ns_param   maxinput           [expr 20 * 1024 * 1024] ;# Maximum File Size for uploads in bytes
-    ns_param   recvwait           [expr 5 * 60] ;# Maximum request time in minutes
+    ns_param   maxinput           [expr {$max_file_upload_mb * 1024 * 1024}] ;# Maximum File Size for uploads in bytes
+    ns_param   maxpost            [expr {$max_file_upload_mb * 1024 * 1024}] ;# Maximum File Size for uploads in bytes
+    ns_param   recvwait           [expr {$max_file_upload_min * 60}] ;# Maximum request time in minutes
 
+# maxsock will limit the number of simultanously returned pages,
+# regardless of what maxthreads is saying
+    ns_param   maxsock               100 ;# 100 = default
+
+# On Windows you need to set this parameter to define the number of
+# connections as well (it seems).
+    ns_param   backlog               5  ;# if < 1 == 5 
+
+# Optional params with defaults:
+    ns_param   bufsize               16000
+    ns_param   rcvbuf                0
+    ns_param   sndbuf                0
+    ns_param   socktimeout           30 ;# if < 1 == 30
+    ns_param   sendwait              30 ;# if < 1 == socktimeout
+    ns_param   recvwait              30 ;# if < 1 == socktimeout
+    ns_param   closewait             2  ;# if < 0 == 2
+    ns_param   keepwait              30 ;# if < 0 == 30
+    ns_param   readtimeoutlogging    false
+    ns_param   serverrejectlogging   false
+    ns_param   sockerrorlogging      false
+    ns_param   sockshuterrorlogging  false
 
 #---------------------------------------------------------------------
 # 
@@ -288,8 +305,8 @@ ns_section ns/server/${server}/module/nsjava
 # 
 #---------------------------------------------------------------------
 #ns_section "ns/server/${server}/module/nscgi" 
-#       ns_param   map "GET  /cgi-bin/ ${serverroot}/cgi-bin"
-#       ns_param   map "POST /cgi-bin/ ${serverroot}/cgi-bin" 
+#       ns_param   map "GET  /cgi-bin ${serverroot}/cgi-bin"
+#       ns_param   map "POST /cgi-bin ${serverroot}/cgi-bin" 
 #       ns_param   Interps CGIinterps
 
 #ns_section "ns/interps/CGIinterps" 
@@ -307,186 +324,118 @@ ns_section ns/server/${server}/module/nspam
 
 #---------------------------------------------------------------------
 #
-# OpenSSL for Aolserver 3.3 and 4
-#
+# OpenSSL for Aolserver  4
+# 
 #---------------------------------------------------------------------
-if { [ns_info version] < 4} {
 
-    #---------------------------------------------------------------------
-    # OpenSSL for Aolserver 3.3
-    #---------------------------------------------------------------------
+ns_section "ns/server/${server}/module/nsopenssl"
 
-    ns_section "ns/server/${server}/module/nsopenssl"
-    
-        ns_param ModuleDir            ${serverroot}/etc/certs
-    
-        # NSD-driven connections:
-        ns_param ServerPort                $httpsport
-        ns_param ServerHostname            $hostname
-        ns_param ServerAddress             $address
-        ns_param ServerCertFile            certfile.pem
-        ns_param ServerKeyFile             keyfile.pem
-        ns_param ServerProtocols           "SSLv2, SSLv3, TLSv1"
-        ns_param ServerCipherSuite         "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP"
-        ns_param ServerSessionCache        true
-        ns_param ServerSessionCacheID      1
-        ns_param ServerSessionCacheSize    512
-        ns_param ServerSessionCacheTimeout 300
-        ns_param ServerPeerVerify          false
-        ns_param ServerPeerVerifyDepth     3
-        ns_param ServerCADir               ca
-        ns_param ServerCAFile              ca.pem
-        ns_param ServerTrace               false
-    
-        # For listening and accepting SSL connections via Tcl/C API:
-        ns_param SockServerCertFile              certfile.pem
-        ns_param SockServerKeyFile               keyfile.pem
-        ns_param SockServerProtocols             "SSLv2, SSLv3, TLSv1"
-        ns_param SockServerCipherSuite           "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP"
-        ns_param SockServerSessionCache          true
-        ns_param SockServerSessionCacheID        2
-        ns_param SockServerSessionCacheSize      512
-        ns_param SockServerSessionCacheTimeout   300
-        ns_param SockServerPeerVerify            false
-        ns_param SockServerPeerVerifyDepth       3
-        ns_param SockServerCADir                 internal_ca
-        ns_param SockServerCAFile                internal_ca.pem
-        ns_param SockServerTrace                 false
-    
-        # Outgoing SSL connections
-        ns_param SockClientCertFile              certfile.pem
-        ns_param SockClientKeyFile               keyfile.pem
-        ns_param SockClientProtocols             "SSLv2, SSLv3, TLSv1"
-        ns_param SockClientCipherSuite           "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP"
-        ns_param SockClientSessionCache          false
-        ns_param SockClientSessionCacheID        3
-        ns_param SockClientSessionCacheSize      512
-        ns_param SockClientSessionCacheTimeout   300
-        ns_param SockClientPeerVerify            false
-        ns_param SockServerPeerVerifyDepth       3
-        ns_param SockClientCADir                 ca
-        ns_param SockClientCAFile                ca.pem
-        ns_param SockClientTrace                 false
-    
-        # OpenSSL library support:
-    #    ns_param RandomFile          /some/file
-        ns_param SeedBytes            1024
+    # this is used by acs-tcl/tcl/security-procs.tcl to get the https port.
+    ns_param ServerPort                $httpsport
+    # setting maxinput higher than practical may leave the server vulnerable to resource DoS attacks
+    # see http://www.panoptic.com/wiki/aolserver/166
+    # must set maxinput for nsopenssl as well as nssock
+    ns_param   maxinput           [expr {$max_file_upload_mb * 1024 * 1024}] ;# Maximum File Size for uploads in bytes
 
-} else {
-
-    #---------------------------------------------------------------------
-    # OpenSSL for Aolserver 4
-    #---------------------------------------------------------------------    
-    ns_section "ns/server/${server}/module/nsopenssl"
-
-    # Note this portion of the configuration is not perfect, and you
-    # will get errors in the your error.log. However, it does
-    # work. Fixes welcome.
-
-        # this is used by acs-tcl/tcl/security-procs.tcl to get the 
-        # https port.
-        ns_param ServerPort                $httpsport
-    
     # We explicitly tell the server which SSL contexts to use as defaults when an
     # SSL context is not specified for a particular client or server SSL
     # connection. Driver connections do not use defaults; they must be explicitly
     # specificied in the driver section. The Tcl API will use the defaults as there
     # is currently no provision to specify which SSL context to use for a
     # particular connection via an ns_openssl Tcl command.
-    ns_section "ns/server/${server}/module/nsopenssl/sslcontexts"
-        ns_param users        "SSL context used for regular user access"
+ns_section "ns/server/${server}/module/nsopenssl/sslcontexts"
+    ns_param users        "SSL context used for regular user access"
     #    ns_param admins       "SSL context used for administrator access"
-        ns_param client       "SSL context used for outgoing script socket connections"
+    ns_param client       "SSL context used for outgoing script socket connections"
 
-    ns_section "ns/server/${server}/module/nsopenssl/defaults"
-        ns_param server               users
-        ns_param client               client
+ns_section "ns/server/${server}/module/nsopenssl/defaults"
+    ns_param server               users
+    ns_param client               client
     
-    ns_section "ns/server/${server}/module/nsopenssl/sslcontext/users"
-        ns_param Role                  server
-        ns_param ModuleDir             ${serverroot}/etc/certs
-        ns_param CertFile              certfile.pem 
-        ns_param KeyFile               keyfile.pem
-    #    ns_param CADir                 ca-client/dir
-    #    ns_param CAFile                ca-client/ca-client.crt
-        # for Protocols                "ALL" = "SSLv2, SSLv3, TLSv1"
-        ns_param Protocols             "SSLv3, TLSv1" 
-        ns_param CipherSuite           "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP" 
-        ns_param PeerVerify            false
-        ns_param PeerVerifyDepth       3
-        ns_param Trace                 false
+ns_section "ns/server/${server}/module/nsopenssl/sslcontext/users"
+    ns_param Role                  server
+    ns_param ModuleDir             ${serverroot}/etc/certs
+    ns_param CertFile              users-certfile.pem 
+    ns_param KeyFile               users-keyfile.pem
+    # CADir/CAFile can be commented out, if CA chain cert is appended to CA issued server cert.
+    ns_param CADir                 ${serverroot}/etc/certs
+    ns_param CAFile                users-ca.crt
+    # for Protocols                "ALL" = "SSLv2, SSLv3, TLSv1"
+    ns_param Protocols             "SSLv3, TLSv1" 
+    ns_param CipherSuite           "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP" 
+    ns_param PeerVerify            false
+    ns_param PeerVerifyDepth       3
+    ns_param Trace                 false
     
-	# following from bartt's nsd4.tcl, might help stablize openssl connections? 
-        # http://www.mail-archive.com/aolserver@listserv.aol.com/msg07092.html
-        ns_param SessionCache true
-        ns_param SessionCacheID 1
-        ns_param SessionCacheSize 512
-        ns_param SessionCacheTimeout 300
+    # following helps to stablize some openssl connections from buggy clients.
+    ns_param SessionCache true
+    ns_param SessionCacheID 1
+    ns_param SessionCacheSize 512
+    ns_param SessionCacheTimeout 300
 
 
 #    ns_section "ns/server/${server}/module/nsopenssl/sslcontext/admins"
-    #    ns_param Role                  server
-    #    ns_param ModuleDir             /path/to/dir
-    #    ns_param CertFile              server/server.crt 
-    #    ns_param KeyFile               server/server.key 
-    #    ns_param CADir                 ca-client/dir 
-    #    ns_param CAFile                ca-client/ca-client.crt
-        # for Protocols                "ALL" = "SSLv2, SSLv3, TLSv1"
-    #    ns_param Protocols             "All"
-    #    ns_param CipherSuite           "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP" 
-    #    ns_param PeerVerify            false
-    #    ns_param PeerVerifyDepth       3
-    #    ns_param Trace                 false
+#    ns_param Role                  server
+#    ns_param ModuleDir             /path/to/dir
+#    ns_param CertFile              server/server.crt 
+#    ns_param KeyFile               server/server.key 
+#    ns_param CADir                 ca-client/dir 
+#    ns_param CAFile                ca-client/ca-client.crt
+    # for Protocols                "ALL" = "SSLv2, SSLv3, TLSv1"
+#    ns_param Protocols             "All"
+#    ns_param CipherSuite           "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP" 
+#    ns_param PeerVerify            false
+#    ns_param PeerVerifyDepth       3
+#    ns_param Trace                 false
     
-    ns_section "ns/server/${server}/module/nsopenssl/sslcontext/client"
-        ns_param Role                  client
-        ns_param ModuleDir             ${serverroot}/etc/certs
-        ns_param CertFile              certfile.pem
-        ns_param KeyFile               keyfile.pem 
-    #    ns_param CADir                 ${serverroot}/etc/certs
-    #    ns_param CAFile                certfile.pem
-        # for Protocols                "ALL" = "SSLv2, SSLv3, TLSv1"
-        ns_param Protocols             "SSLv2, SSLv3, TLSv1" 
-        ns_param CipherSuite           "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP" 
-        ns_param PeerVerify            false
-        ns_param PeerVerifyDepth       3
-        ns_param Trace                 false
-    
-	# following from bartt's nsd4.tcl, might help stablize openssl connections? 
-        # http://www.mail-archive.com/aolserver@listserv.aol.com/msg07092.html
-        ns_param SessionCache true
-        ns_param SessionCacheID 1
-        ns_param SessionCacheSize 512
-        ns_param SessionCacheTimeout 300
+ns_section "ns/server/${server}/module/nsopenssl/sslcontext/client"
+    ns_param Role                  client
+    ns_param ModuleDir             ${serverroot}/etc/certs
+    ns_param CertFile              client-certfile.pem
+    ns_param KeyFile               client-keyfile.pem 
+    # CADir/CAFile can be commented out, if CA chain cert is appended to CA issued server cert.
+    ns_param CADir                 ${serverroot}/etc/certs
+    ns_param CAFile                client-ca.crt
+    # for Protocols                "ALL" = "SSLv2, SSLv3, TLSv1"
+    ns_param Protocols             "SSLv2, SSLv3, TLSv1" 
+    ns_param CipherSuite           "ALL:!ADH:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP" 
+    ns_param PeerVerify            false
+    ns_param PeerVerifyDepth       3
+    ns_param Trace                 false
 
-    # SSL drivers. Each driver defines a port to listen on and an explitictly named
-    # SSL context to associate with it. Note that you can now have multiple driver
-    # connections within a single virtual server, which can be tied to different
-    # SSL contexts.
-    ns_section "ns/server/${server}/module/nsopenssl/ssldrivers"
-        ns_param users         "Driver for regular user access"
-    #    ns_param admins        "Driver for administrator access"
-    
-    ns_section "ns/server/${server}/module/nsopenssl/ssldriver/users"
-        ns_param sslcontext            users
-        # ns_param port                  $httpsport_users
-        ns_param port                  $httpsport
-        ns_param hostname              $hostname
-        ns_param address               $address
-        # following added per
-        # http://www.mail-archive.com/aolserver@listserv.aol.com/msg07365.html
-        # Maximum File Size for uploads:
-        ns_param   maxinput           [expr 5 * 1024 * 1024] ;# in bytes
-        # Maximum request time
-        ns_param   recvwait           [expr 5 * 60] ;# in minutes
+# following helps to stablize some openssl connections to buggy servers.
+    ns_param SessionCache true
+    ns_param SessionCacheID 1
+    ns_param SessionCacheSize 512
+    ns_param SessionCacheTimeout 300
+
+# SSL drivers. Each driver defines a port to listen on and an explitictly named
+# SSL context to associate with it. Note that you can now have multiple driver
+# connections within a single virtual server, which can be tied to different
+# SSL contexts.
+ns_section "ns/server/${server}/module/nsopenssl/ssldrivers"
+    ns_param users         "Driver for regular user access"
+#    ns_param admins        "Driver for administrator access"
+
+ns_section "ns/server/${server}/module/nsopenssl/ssldriver/users"
+    ns_param sslcontext            users
+    # ns_param port                  $httpsport_users
+    ns_param port                  $httpsport
+    ns_param hostname              $hostname
+    ns_param address               $address
+    # following added per
+    # http://www.mail-archive.com/aolserver@listserv.aol.com/msg07365.html
+    # Maximum File Size for uploads:
+    ns_param   maxinput           [expr {$max_file_upload_mb * 1024 * 1024}] ;# in bytes
+    # Maximum request time
+    ns_param   recvwait           [expr {$max_file_upload_min * 60}] ;# in minutes
 
 #    ns_section "ns/server/${server}/module/nsopenssl/ssldriver/admins"
-    #    ns_param sslcontext            admins
-    #    ns_param port                  $httpsport_admins
-    #    ns_param port                  $httpsport
-    #    ns_param hostname              $hostname
-    #    ns_param address               $address
-}
+#    ns_param sslcontext            admins
+#    ns_param port                  $httpsport_admins
+#    ns_param port                  $httpsport
+#    ns_param hostname              $hostname
+#    ns_param address               $address
 
 
 #---------------------------------------------------------------------
@@ -497,17 +446,17 @@ if { [ns_info version] < 4} {
 #
 #---------------------------------------------------------------------
 ns_section "ns/db/drivers" 
-if { $database == "oracle" } {
-    ns_param   ora8           ${bindir}/ora8.so
-} else {
-    ns_param   postgres       ${bindir}/nspostgres.so  ;# Load PostgreSQL driver
-}
+    if { $database eq "oracle" } {
+        ns_param   ora8           ${bindir}/ora8.so
+    } else {
+        ns_param   postgres       ${bindir}/nspostgres.so  ;# Load PostgreSQL driver
+    }
 
-if { $database == "oracle" } {
-    ns_section "ns/db/driver/ora8"
-    ns_param  maxStringLogLength -1
-    ns_param  LobBufferSize      32768
-}
+    if { $database eq "oracle" } {
+        ns_section "ns/db/driver/ora8"
+        ns_param  maxStringLogLength -1
+        ns_param  LobBufferSize      32768
+    }
 
  
 # Database Pools: This is how AOLserver  ``talks'' to the RDBMS. You need 
@@ -517,21 +466,26 @@ if { $database == "oracle" } {
 #
 # AOLserver can have different pools connecting to different databases 
 # and even different different database servers.  See
-# http://openacs.org/doc/openacs-5-1/tutorial-second-database.html
+# http://openacs.org/doc/tutorial-second-database.html
+# An example 'other db' configuration is included (and commented out) using other1_db_name
+# set other1_db_name "yourDBname"
 
 ns_section ns/db/pools 
     ns_param   pool1              "Pool 1"
     ns_param   pool2              "Pool 2"
     ns_param   pool3              "Pool 3"
+#    ns_param   pool4              "Pool4 Other1"
+#    ns_param   pool5              "Pool5 Other1"
+#    ns_param   pool6              "Pool6 Other1"
 
 ns_section ns/db/pool/pool1
-    ns_param   maxidle            1000000000
-    ns_param   maxopen            1000000000
-    ns_param   connections        5
+    ns_param   maxidle            0
+    ns_param   maxopen            0
+    ns_param   connections        15
     ns_param   verbose            $debug
     ns_param   extendedtableinfo  true
     ns_param   logsqlerrors       $debug
-    if { $database == "oracle" } {
+    if { $database eq "oracle" } {
         ns_param   driver             ora8
         ns_param   datasource         {}
         ns_param   user               $db_name
@@ -544,13 +498,13 @@ ns_section ns/db/pool/pool1
     } 
 
 ns_section ns/db/pool/pool2
-    ns_param   maxidle            1000000000
-    ns_param   maxopen            1000000000
+    ns_param   maxidle            0
+    ns_param   maxopen            0
     ns_param   connections        5
     ns_param   verbose            $debug
     ns_param   extendedtableinfo  true
     ns_param   logsqlerrors       $debug
-    if { $database == "oracle" } {
+    if { $database eq "oracle" } {
         ns_param   driver             ora8
         ns_param   datasource         {}
         ns_param   user               $db_name
@@ -563,13 +517,13 @@ ns_section ns/db/pool/pool2
     } 
 
 ns_section ns/db/pool/pool3
-    ns_param   maxidle            1000000000
-    ns_param   maxopen            1000000000
+    ns_param   maxidle            0
+    ns_param   maxopen            0
     ns_param   connections        5
     ns_param   verbose            $debug
     ns_param   extendedtableinfo  true
     ns_param   logsqlerrors       $debug
-    if { $database == "oracle" } {
+    if { $database eq "oracle" } {
         ns_param   driver             ora8
         ns_param   datasource         {}
         ns_param   user               $db_name
@@ -581,10 +535,46 @@ ns_section ns/db/pool/pool3
         ns_param   password           ""
     } 
 
+# ns_section ns/db/pool/pool4
+#    ns_param   maxidle            0
+#    ns_param   maxopen            0
+#    ns_param   connections        5
+#    ns_param   verbose            $debug
+#    ns_param   extendedtableinfo  true
+#    ns_param   logsqlerrors       $debug
+#    if { $database eq "oracle" } {
+#        ns_param   driver             ora8
+#        ns_param   datasource         {}
+#        ns_param   user               $db_name
+#        ns_param   password           $db_password
+#    } else {
+#        ns_param   driver             postgres 
+#        ns_param   datasource         ${db_host}:${db_port}:${other1_db_name}
+#        ns_param   user               $db_user
+#        ns_param   password           ""
+#    } 
+
+# ns_section ns/db/pool/pool5
+# ...
+# ns_section ns/db/pool/pool6
+# ...
+
+
 ns_section ns/server/${server}/db
-    ns_param   pools              "*" 
+    ns_param   pools              pool1,pool2,pool3
+# if a second db is added, add the pools here. for example, replace above line with:
+#    ns_param   pools              pool1,pool2,pool3,pool4,pool5,pool6
     ns_param   defaultpool        pool1
 
+# following from http://openacs.org/doc/tutorial-second-database.html
+#ns_section ns/server/${server}/acs/database
+#    ns_param database_names [list main other1]
+#    ns_param pools_main [list pool1 pool2 pool3]
+#    ns_param pools_other1 [list pool4 pool5 pool6]
+# Start each pool set with pools_* 
+# The code assumes the name in database_names matches the suffix to pools_ in one of the ns_params.
+
+
 
 #---------------------------------------------------------------------
 # which modules should be loaded?  Missing modules break the server, so
@@ -593,8 +583,11 @@ ns_section ns/server/${server}/modules
     ns_param   nssock             ${bindir}/nssock.so 
     ns_param   nslog              ${bindir}/nslog.so 
     ns_param   nssha1             ${bindir}/nssha1.so 
-    ns_param   nscache            ${bindir}/nscache.so 
-
+    # since aolserver version 4.5.1 built-in ns_cache, so we dont
+    # need to load the nscache module. 
+    if {[ns_info version] < 4.5 || [ns_info patchlevel] eq "4.5.0"} {
+          ns_param   nscache            ${bindir}/nscache.so 
+    }
     # openacs versions earlier than 5.x requires nsxml
 #    ns_param nsxml              ${bindir}/nsxml.so
 
@@ -629,6 +622,12 @@ ns_section ns/server/${server}/modules
         ns_param   libtdom            ${bindir}/libtdom.so
     }
 
+    # nsthread library which should become standard in 5.3
+    ns_param libthread  [lindex [glob ${homedir}/lib/thread*/libthread*[info sharedlibextension]] 0]
+
+    if {[ns_info version] >= 4.5} {
+        ns_limits set default -maxupload [ns_config ns/server/${server}/module/nssock maxinput]
+    }
 
 ns_log notice "nsd.tcl: using threadsafe tcl: [info exists tcl_platform(threaded)]"
 ns_log notice "nsd.tcl: finished reading config file."
