@@ -2,7 +2,7 @@ ad_page_contract {
     Generates package specs for every enabled version.
     @author Jon Salz (jsalz@arsdigita.com)
     @creation-date 17 April 2000
-    @cvs-id $Id: write-all-specs.tcl,v 1.5 2003/03/10 19:59:31 peterm Exp $
+    @cvs-id $Id: write-all-specs.tcl,v 1.6 2007/01/10 21:21:59 gustafn Exp $
 } {
 }
 
@@ -20,7 +20,7 @@ db_foreach apm_get_all_packages {
     and v.package_key = t.package_key
     order by upper(pretty_name)
 } {
-    if { [empty_string_p $distribution_uri] } {
+    if { $distribution_uri eq "" } {
 	ns_log Debug "Generating package specificaiton for $package_key"
 	ns_write "<li>$pretty_name $version_name... "
 	if { [catch { 
