@@ -3,7 +3,7 @@ ad_page_contract {
 
     @author Hiro Iwashima <iwashima@mit.edu>
     @creation-date 23 Aug 2000
-    @cvs-id $Id: send-email.tcl,v 1.2 2002/11/30 17:12:49 jeffd Exp $
+    @cvs-id $Id: send-email.tcl,v 1.4 2009/07/12 01:08:22 donb Exp $
 } {
     email
     email_from
@@ -23,13 +23,13 @@ ad_page_contract {
     return_url:onevalue
 }
 
-if [catch {ns_sendmail $email $email_from $subject $message} errmsg] {
-    ad_return_error $error_subject "$error_message:
-    <blockquote><pre>[ad_quotehtml $errmsg]</pre></blockquote>"
+if {[catch {acs_mail_lite::send -send_immediately -to_addr $email -from_addr $email_from -subject $subject -body $message} errmsg]} {
+    ad_return_error $error_subject "<p>$error_message</p>
+    <div><code>[ad_quotehtml $errmsg]</code></div>"
     ad_script_abort
 }
 
-if { $show_sent_message_p != "t" } {
+if { $show_sent_message_p ne "t" } {
     # Do not show any message. Just go to return url
     ad_returnredirect $return_url
     ad_script_abort
