@@ -3,7 +3,7 @@ ad_library {
 
     @author Peter Marklund
     @creation-date 20 April 2004
-    @cvs-id $Id: acs-automated-testing-procs.tcl,v 1.1.2.1 2004/04/20 14:06:57 peterm Exp $
+    @cvs-id $Id: acs-automated-testing-procs.tcl,v 1.3 2006/10/29 19:07:12 donb Exp $
 }
 
 aa_register_case -cats {web smoke} -libraries tclwebtest tclwebtest_example {
@@ -24,6 +24,9 @@ aa_register_case -cats {web smoke} -libraries tclwebtest tclwebtest_example {
             # Visit homepage
             twt::do_request "/"
 
+	    # Logout user
+            twt::user::logout
+
         } -teardown_code {
             # TODO: delete test user
             twt::user::delete -user_id $user_id
