@@ -4,31 +4,12 @@
 -- Copyright (C) 1999-2000 ArsDigita Corporation
 -- Author: Karl Goldstein (karlg@arsdigita.com)
 
--- $Id: content-create.sql,v 1.21.4.1 2004/09/06 16:43:47 rocaelh Exp $
+-- $Id: content-create.sql,v 1.29 2011/04/11 01:08:27 donb Exp $
 
 -- This is free software distributed under the terms of the GNU Public
 -- License.  Full text of the license is available from the GNU Project:
 -- http://www.fsf.org/copyleft/gpl.html
 
-----------------------------------
--- CMS datatypes
-----------------------------------
-
--- create ats datatypes for cms
-begin
-  insert into acs_datatypes
-    (datatype, max_n_values)
-  values
-    ('text', null);
-
-  insert into acs_datatypes
-    (datatype, max_n_values)
-  values
-    ('keyword', 1);
-
-end;
-/
-
 
 --------------------------------------------------------------
 -- MIME TYPES
@@ -102,12 +83,12 @@ create table cr_locales (
                         constraint cr_locale_abbrev_pk
                         primary key,
   label			varchar2(200)
-                        constraint cr_locale_name_nil
+                        constraint cr_locale_name_nn
 			not null
-                        constraint cr_locale_name_unq
+                        constraint cr_locale_name_un
                         unique,
   nls_language		varchar2(30)
-                        constraint cr_locale_nls_lang_nil
+                        constraint cr_locale_nls_lang_nn
 			not null,
   nls_territory		varchar2(30),
   nls_charset		varchar2(30)
@@ -191,7 +172,7 @@ create table cr_items (
                       constraint cr_items_parent_id_fk references
                       acs_objects on delete cascade,
   name                varchar2(400)
-                      constraint cr_items_name_nil
+                      constraint cr_items_name_nn
                       not null,
   locale              varchar2(4)
                       constraint cr_items_locale_fk references
@@ -199,7 +180,7 @@ create table cr_items (
   live_revision       integer,
   latest_revision     integer,
   publish_status      varchar2(40) 
-                      constraint cr_items_pub_status_chk
+                      constraint cr_items_pub_status_ck
                       check (publish_status in 
                             ('production', 'ready', 'live', 'expired')
                       ),
@@ -207,7 +188,7 @@ create table cr_items (
                       constraint cr_items_rev_type_fk
                       references acs_object_types,
   storage_type        varchar2(10) default 'lob' not null
-                      constraint cr_items_storage_type
+                      constraint cr_items_storage_type_ck
                       check (storage_type in ('lob','file')),
   storage_area_key    varchar2(100) default 'CR_FILES' not null
 );  
@@ -262,10 +243,10 @@ create table cr_child_rels (
                      constraint cr_child_rels_rel_fk
                      references acs_objects,
   parent_id          integer
-                     constraint cr_child_rels_parent_nil
+                     constraint cr_child_rels_parent_nn
                      not null,
   child_id           integer
-                     constraint cr_child_rels_child_nil
+                     constraint cr_child_rels_child_nn
                      not null,
   relation_tag       varchar2(100),
   order_n            integer
@@ -273,7 +254,8 @@ create table cr_child_rels (
 
 create index cr_child_rels_by_parent on cr_child_rels(parent_id);
 create unique index cr_child_rels_unq_id on cr_child_rels(parent_id, child_id);
-
+CREATE UNIQUE INDEX CR_CHILD_RELS_kids_IDx ON CR_CHILD_RELS(CHILD_ID);
+    
 comment on table cr_child_rels is '
   Provides for richer parent-child relationships than the simple
   link encapsulated in the primary table.  May be subclassed to provide
@@ -290,7 +272,7 @@ create table cr_item_rels (
                      constraint cr_item_rels_item_fk
                      references cr_items,
   related_object_id  integer
-                     constraint cr_item_rels_rel_obj__fk
+                     constraint cr_item_rels_rel_obj_fk
                      references acs_objects,
   relation_tag       varchar2(100),
   order_n            integer
@@ -330,7 +312,7 @@ create table cr_revisions (
   revision_id     constraint cr_revisions_rev_id_fk references
 		  acs_objects (object_id) on delete cascade
 		  constraint cr_revisions_pk primary key,
-  item_id         constraint cr_revisions_item_id_nil
+  item_id         constraint cr_revisions_item_id_nn
                   not null
                   constraint cr_revisions_item_id_fk references
 		  cr_items on delete cascade,
@@ -338,7 +320,7 @@ create table cr_revisions (
   description	  varchar2(4000),
   publish_date	  date,
   mime_type	  varchar2(200) default 'text/plain'
-		  constraint cr_revisions_mime_type_ref
+		  constraint cr_revisions_mime_type_fk
 		  references cr_mime_types,
   nls_language    varchar2(50),
   filename        varchar2(4000),
@@ -350,10 +332,13 @@ create table cr_revisions (
 create index cr_revisions_by_mime_type on cr_revisions(mime_type);
 create index cr_revisions_title_idx on cr_revisions(title);
 create index cr_revisions_item_id_idx ON cr_revisions(item_id);
+create index cr_revisions_publish_date_idx on cr_revisions(publish_date);
 
 -- create index cr_revisions_lower_title_idx on cr_revisions(lower(title));
 -- create index cr_revisions_title_ltr_idx on cr_revisions(substr(lower(title), 1, 1));
 
+
+
 comment on table cr_revisions is '
   Each content item may be associated with any number of revisions.
   The item_id is used to associate revisions with an item.
@@ -384,7 +369,9 @@ comment on table cr_revision_attributes is '
 ';
 
 create global temporary table cr_content_text (
-    revision_id        integer primary key,
+    revision_id        integer 
+		       constraint cr_content_text_revision_id_pk 
+		       primary key,
     content            CLOB
 ) on commit delete rows;
 
@@ -411,7 +398,7 @@ create table cr_item_publish_audit (
   old_status         varchar2(40),
   new_status         varchar2(40),
   publish_date       date
-                     constraint cr_item_publish_audit_date_nil
+                     constraint cr_item_publish_audit_date_nn
                      not null
 );
 
@@ -462,7 +449,7 @@ insert into cr_scheduled_release_job values (NULL, sysdate);
 
 create table cr_folders (
   folder_id	    integer
-		    constraint cr_folder_id_fk references
+		    constraint cr_folders_folder_id_fk references
 		    cr_items on delete cascade
 		    constraint cr_folders_pk 
                     primary key,
@@ -470,11 +457,11 @@ create table cr_folders (
   description	    varchar2(4000),
   has_child_folders char(1)
                     default 'f'
-                    constraint cr_folder_child_chk
+                    constraint cr_folder_child_ck
                     check (has_child_folders in ('t','f')),
   has_child_symlinks char(1)
                      default 'f'
-                     constraint cr_folder_symlink_chk
+                     constraint cr_folder_symlink_ck
                      check (has_child_symlinks in ('t', 'f')),
   package_id        integer 
                     constraint cr_fldr_pkg_id_fk references apm_packages
@@ -545,13 +532,13 @@ create table cr_type_template_map (
   content_type     varchar2(100)
                    constraint cr_type_template_map_typ_fk
                    references acs_object_types
-                   constraint cr_type_template_map_typ_nil
+                   constraint cr_type_template_map_typ_nn
                    not null,
   template_id      integer
                    constraint cr_type_template_map_tmpl_fk
 	           references cr_templates,
   use_context	   varchar2(100)
-                   constraint cr_type_template_map_ctx_nil
+                   constraint cr_type_template_map_ctx_nn
                    not null
                    constraint cr_type_template_map_ctx_fk
                    references cr_template_use_contexts,
@@ -584,7 +571,7 @@ create table cr_item_template_map (
   item_id          integer
                    constraint cr_item_template_map_item_fk
                    references cr_items
-                   constraint cr_item_template_map_item_nil
+                   constraint cr_item_template_map_item_nn
                    not null,
   template_id      integer
                    constraint cr_item_template_map_tmpl_fk
@@ -592,7 +579,7 @@ create table cr_item_template_map (
                    constraint cr_item_template_map_tmpl_nil
                    not null,
   use_context	   varchar2(100)
-                   constraint cr_item_template_map_ctx_nil
+                   constraint cr_item_template_map_ctx_nn
                    not null
                    constraint cr_item_template_map_ctx_fk
                    references cr_template_use_contexts,
@@ -621,7 +608,7 @@ create table cr_symlinks (
   target_id       integer
                   constraint cr_symlink_target_id_fk
 		  references cr_items
-		  constraint cr_symlink_target_id_nil
+		  constraint cr_symlink_target_id_nn
 		  not null,
   label		  varchar2(1000)
 );
@@ -643,10 +630,10 @@ create table cr_extlinks (
 		  constraint cr_extlinks_pk 
                   primary key,
   url             varchar2(1000)
-		  constraint cr_extlink_url_nil
+		  constraint cr_extlink_url_nn
 		  not null,
   label           varchar2(1000)
-		  constraint cr_extlink_label_nil
+		  constraint cr_extlink_label_nn
 		  not null,
   description	  varchar2(4000)
 );
@@ -665,14 +652,14 @@ create table cr_keywords (
 			 constraint cr_keywords_pk
 		         primary key,
   parent_id              integer 
-                         constraint cr_keywords_hier
+                         constraint cr_keywords_fk
                          references cr_keywords,
   heading		 varchar2(600)
-			 constraint cr_keywords_name_nil
+			 constraint cr_keywords_name_nn
 			 not null,
   description            varchar2(4000),
   has_children           char(1)
-                         constraint cr_keywords_child_chk
+                         constraint cr_keywords_child_ck
                          check (has_children in ('t', 'f'))
 );
 
@@ -700,12 +687,12 @@ create table cr_item_keyword_map (
   item_id          integer
                    constraint cr_item_keyword_map_item_fk
                    references cr_items
-                   constraint cr_item_keyword_map_item_nil
+                   constraint cr_item_keyword_map_item_nn
                    not null,
   keyword_id       integer
                    constraint cr_item_keyword_map_kw_fk
 	           references cr_keywords
-                   constraint cr_item_keyword_map_kw_nil
+                   constraint cr_item_keyword_map_kw_nn
                    not null,
   constraint cr_item_keyword_map_pk
   primary key (item_id, keyword_id)
@@ -747,12 +734,6 @@ show errors
 -- DOCUMENT SUBMISSION WITH CONVERSION TO HTML
 --------------------------------------------------------------
 
-create global temporary table cr_doc_filter (
-    revision_id        integer primary key,
-    content            BLOB
-) on commit delete rows;
-
-
 -- Source PL/SQL Definitions.
 
 @@ content-util
@@ -770,16 +751,6 @@ prompt *** Creating object types...
 prompt *** Preparing search indices...
 @@ content-search
 
--- this index requires prefs created in content-search
-create index cr_doc_filter_index on cr_doc_filter ( content )
-  indextype is ctxsys.context
-  parameters ('FILTER content_filter_pref' );
-
-comment on table cr_doc_filter is '
-  A temporary table for holding binary documents that are to be converted
-  into HTML (or plain text) prior to insertion into the repository.
-';
-
 -- (DanW - OpenACS) Added cleanup trigger to log file items that need 
 -- to be cleaned up from the CR.
 
@@ -832,7 +803,7 @@ begin
     name        => 'pages',
     label       => 'Pages', 
     description => 'Site pages go here',
-    parent_id   => 0,
+    parent_id   => -4,
     folder_id   => content_item.get_root_folder
   );
 
@@ -854,11 +825,17 @@ begin
     include_subtypes => 't'
   );
 
+  -- add the root content folder to acs_magic_objects
+  insert into acs_magic_objects (name, object_id)
+  select 'cr_item_root',
+         content_item.get_root_folder
+    from dual;
+
   v_id := content_folder.new (
     name        => 'templates',
     label       => 'Templates', 
     description => 'Templates which render the pages go here',
-    parent_id   => 0,
+    parent_id   => -4,
     folder_id   => content_template.get_root_folder
   );
 
@@ -880,6 +857,12 @@ begin
     include_subtypes => 't'
   );
 
+  -- add to acs_magic_objects
+  insert into acs_magic_objects (name, object_id)
+  select 'cr_template_root',
+         content_template.get_root_folder
+    from dual;
+
 end;
 /
 show errors
