@@ -4,7 +4,7 @@
 -- Copyright (C) 1999-2000 ArsDigita Corporation
 -- Author: Karl Goldstein (karlg@arsdigita.com)
 
--- $Id: content-folder.sql,v 1.33.2.6 2005/01/14 03:14:45 daveb Exp $
+-- $Id: content-folder.sql,v 1.50 2008/10/24 13:50:14 victorg Exp $
 
 -- This is free software distributed under the terms of the GNU Public
 -- License.  Full text of the license is available from the GNU Project:
@@ -12,6 +12,30 @@
 
 -- create or replace package body content_folder
 
+
+create or replace function content_folder__new(varchar,varchar,varchar,integer,integer) 
+returns integer as '
+declare
+  new__name                   alias for $1;  
+  new__label                  alias for $2;  
+  new__description            alias for $3;  -- default null  
+  new__parent_id              alias for $4;  -- default null
+  new__package_id             alias for $5;  -- default null
+begin
+        return content_folder__new(new__name,
+                                   new__label,
+                                   new__description,
+                                   new__parent_id,
+                                   null,
+                                   null,
+                                   now(),
+                                   null,
+                                   null,
+                                   new__package_id
+               );
+
+end;' language 'plpgsql';
+
 create or replace function content_folder__new(varchar,varchar,varchar,integer) 
 returns integer as '
 declare
@@ -28,10 +52,13 @@ begin
                                    null,
                                    now(),
                                    null,
+                                   null,
+                                   ''t'',
                                    null
                );
 
 end;' language 'plpgsql';
+
 -- function new
 create or replace function content_folder__new (varchar,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar)
 returns integer as '
@@ -45,77 +72,24 @@ declare
   new__creation_date          alias for $7;  -- default now()
   new__creation_user          alias for $8;  -- default null
   new__creation_ip            alias for $9;  -- default null
-  v_folder_id                 cr_folders.folder_id%TYPE;
-  v_context_id                acs_objects.context_id%TYPE;
 begin
+        return content_folder__new(new__name,
+                                   new__label,
+                                   new__description,
+                                   new__parent_id,
+                                   new__context_id,
+                                   new__folder_id,
+                                   new__creation_date,
+                                   new__creation_user,
+                                   new__creation_ip,
+                                   ''t'',
+                                   null::integer
+               );
 
-  -- set the context_id
-  if new__context_id is null then
-    v_context_id := new__parent_id;
-  else
-    v_context_id := new__context_id;
-  end if;
-
-  -- parent_id = 0 means that this is a mount point
-  if new__parent_id != 0 and 
-     content_folder__is_registered(new__parent_id,''content_folder'',''f'') = ''f'' then
-
-    raise EXCEPTION ''-20000: This folder does not allow subfolders to be created'';
-    return null;
-
-  else
-
-    v_folder_id := content_item__new(
-        new__name, 
-        new__parent_id,
-        new__folder_id,
-        null,
-        new__creation_date, 
-        new__creation_user, 
-	v_context_id,
-        new__creation_ip, 
-        ''content_folder'',
-        ''content_folder'',
-        null,
-        null,
-        ''text/plain'',
-        null,
-        null,
-        ''text''    
-    );
-
-    insert into cr_folders (
-      folder_id, label, description
-    ) values (
-      v_folder_id, new__label, new__description
-    );
-
-    -- inherit the attributes of the parent folder
-    if new__parent_id is not null then
-    
-      insert into cr_folder_type_map
-        select
-          v_folder_id as folder_id, content_type
-        from
-          cr_folder_type_map
-        where
-          folder_id = new__parent_id;
-    end if;
-
-    -- update the child flag on the parent
-    update cr_folders set has_child_folders = ''t''
-      where folder_id = new__parent_id;
-
-    return v_folder_id;
-
-  end if;
-
-  return null; 
 end;' language 'plpgsql';
 
--- function new -- accepts security_inherit_p DaveB
-
-create or replace function content_folder__new (varchar,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar, boolean)
+-- function new
+create or replace function content_folder__new (varchar,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar,integer)
 returns integer as '
 declare
   new__name                   alias for $1;  
@@ -127,7 +101,39 @@ declare
   new__creation_date          alias for $7;  -- default now()
   new__creation_user          alias for $8;  -- default null
   new__creation_ip            alias for $9;  -- default null
-  new__security_inherit_p     alias for $10;  -- default true	
+  new__package_id             alias for $10;  -- default null
+  v_folder_id                 cr_folders.folder_id%TYPE;
+  v_context_id                acs_objects.context_id%TYPE;
+  v_package_id                acs_objects.package_id%TYPE;
+begin
+        return content_folder__new(new__name,
+                                   new__label,
+                                   new__description,
+                                   new__parent_id,
+                                   new__context_id,
+                                   new__folder_id,
+                                   new__creation_date,
+                                   new__creation_user,
+                                   new__creation_ip,
+                                   ''t'',
+                                   new__package_id
+               );
+end;' language 'plpgsql';
+
+create or replace function content_folder__new (varchar,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar,boolean)
+returns integer as '
+declare
+  new__name                   alias for $1;
+  new__label                  alias for $2;
+  new__description            alias for $3;  -- default null
+  new__parent_id              alias for $4;  -- default null
+  new__context_id             alias for $5;  -- default null
+  new__folder_id              alias for $6;  -- default null
+  new__creation_date          alias for $7;  -- default now()
+  new__creation_user          alias for $8;  -- default null
+  new__creation_ip            alias for $9;  -- default null
+  new__security_inherit_p     alias for $10;  -- default true
+  v_package_id                acs_objects.package_id%TYPE;
   v_folder_id                 cr_folders.folder_id%TYPE;
   v_context_id                acs_objects.context_id%TYPE;
 begin
@@ -148,6 +154,7 @@ begin
 
 end;' language 'plpgsql';
 
+-- function new -- accepts security_inherit_p DaveB
 select define_function_args('content_folder__new','name,label,description,parent_id,context_id,folder_id,creation_date;now,creation_user,creation_ip,security_inherit_p;t,package_id');
 
 create or replace function content_folder__new (varchar,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar, boolean,integer)
@@ -175,9 +182,10 @@ begin
     v_context_id := new__context_id;
   end if;
 
-  -- parent_id = 0 means that this is a mount point
-  if new__parent_id != 0 and 
-     content_folder__is_registered(new__parent_id,''content_folder'',''f'') = ''f'' then
+  -- parent_id = security_context_root means that this is a mount point
+  if new__parent_id != -4 and 
+    content_folder__is_folder(new__parent_id) and
+    content_folder__is_registered(new__parent_id,''content_folder'',''f'') = ''f'' then
 
     raise EXCEPTION ''-20000: This folder does not allow subfolders to be created'';
     return null;
@@ -200,7 +208,9 @@ begin
 	new__security_inherit_p,
 	''CR_FILES'',
 	''content_folder'',
-        ''content_folder'');
+        ''content_folder'',
+        new__package_id
+    );
 
     insert into cr_folders (
       folder_id, label, description, package_id
@@ -208,6 +218,11 @@ begin
       v_folder_id, new__label, new__description, new__package_id
     );
 
+    -- set the correct object title
+    update acs_objects
+    set title = new__label
+    where object_id = v_folder_id;
+
     -- inherit the attributes of the parent folder
     if new__parent_id is not null then
     
@@ -232,13 +247,42 @@ where
   return v_folder_id; 
 end;' language 'plpgsql';
 
+create or replace function content_folder__new (varchar,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar,boolean)
+returns integer as '
+declare
+  new__name                   alias for $1;  
+  new__label                  alias for $2;  
+  new__description            alias for $3;  -- default null
+  new__parent_id              alias for $4;  -- default null
+  new__context_id             alias for $5;  -- default null
+  new__folder_id              alias for $6;  -- default null
+  new__creation_date          alias for $7;  -- default now()
+  new__creation_user          alias for $8;  -- default null
+  new__creation_ip            alias for $9;  -- default null
+  new__security_inherit_p     alias for $10;  -- default true	
+begin
+        return content_folder__new(new__name,
+                                   new__label,
+                                   new__description,
+                                   new__parent_id,
+                                   new__context_id,
+                                   new__folder_id,
+                                   new__creation_date,
+                                   new__creation_user,
+                                   new__creation_ip,
+                                   new__security_inherit_p,
+                                   null::integer
+               );
+
+end;' language 'plpgsql';
+
 -- procedure delete
 select define_function_args('content_folder__del','folder_id,cascade_p;f');
 create or replace function content_folder__del (integer, boolean)
 returns integer as '
 declare
   delete__folder_id              alias for $1;  
-  p_cascade_p                    alias for $2;
+  p_cascade_p                    alias for $2; -- default ''f''
   v_count                        integer;       
   v_child_row                    record;
   v_parent_id                    integer;  
@@ -303,7 +347,7 @@ create or replace function content_folder__delete (integer, boolean)
 returns integer as '
 declare
   delete__folder_id              alias for $1;  
-  p_cascade_p                    alias for $2;
+  p_cascade_p                    alias for $2;  -- default ''f''
 begin
         PERFORM content_folder__del(delete__folder_id,p_cascade_p);
   return 0; 
@@ -341,6 +385,12 @@ begin
     PERFORM content_item__edit_name(edit_name__folder_id, edit_name__name);
   end if;
 
+  if edit_name__label is not null and edit_name__label != '''' then
+    update acs_objects
+    set title = edit_name__label
+    where object_id = edit_name__folder_id;
+  end if;
+
   if edit_name__label is not null and edit_name__label != '''' and 
      edit_name__description is not null and edit_name__description != '''' then 
 
@@ -369,13 +419,14 @@ end;' language 'plpgsql';
 -- 3) update the parent_id for the folder
 
 -- procedure move
-select define_function_args('content_folder__move','folder_id,target_folder_id');
+select define_function_args('content_folder__move','folder_id,target_folder_id,name;null');
 
-create or replace function content_folder__move (integer,integer)
+create or replace function content_folder__move (integer,integer,varchar)
 returns integer as '
 declare
   move__folder_id              alias for $1;  
-  move__target_folder_id       alias for $2;  
+  move__target_folder_id       alias for $2;
+  move__name                   alias for $3; -- default null
   v_source_folder_id           integer;       
   v_valid_folders_p            integer;
 begin
@@ -417,7 +468,8 @@ begin
 
    -- update the parent_id for the folder
    update cr_items 
-     set parent_id = move__target_folder_id
+     set parent_id = move__target_folder_id,
+         name = coalesce ( move__name, name )
      where item_id = move__folder_id;
 
   -- update the has_child_folders flags
@@ -436,6 +488,20 @@ begin
   return 0; 
 end;' language 'plpgsql';
 
+create or replace function content_folder__move (integer,integer)
+returns integer as '
+declare
+  move__folder_id              alias for $1;  
+  move__target_folder_id       alias for $2;  
+begin
+
+  perform content_folder__move (
+                                move__folder_id,
+                                move__target_folder_id,
+                                NULL
+                               );
+  return null;
+end;' language 'plpgsql';
 
 -- procedure copy
 create or replace function content_folder__copy (integer,integer,integer,varchar)
@@ -457,13 +523,13 @@ begin
 			copy__folder_id,
 			copy__target_folder_id,
 			copy__creation_user,
-			copy_creation_ip,
+			copy__creation_ip,
 			NULL
 			);
 	return v_new_folder_id;
 end;' language 'plpgsql';
 
-create function content_folder__copy (integer,integer,integer,varchar,varchar)
+create or replace function content_folder__copy (integer,integer,integer,varchar,varchar)
 returns integer as '
 declare
   copy__folder_id              alias for $1;  
@@ -480,6 +546,11 @@ declare
   v_folder_contents_val        record;
 begin
 
+  if copy__folder_id = content_item__get_root_folder(null) 
+     or copy__folder_id = content_template__get_root_folder() then
+     raise EXCEPTION ''-20000: content_folder.copy - Not allowed to copy root folder'';
+  end if;
+
   select 
     count(*)
   into 
@@ -491,36 +562,39 @@ begin
   or 
     folder_id = copy__folder_id;
 
-  select
-    parent_id
-  into
-    v_current_folder_id
-  from
-    cr_items
-  where
-    item_id = copy__folder_id;  
-
-  if copy__folder_id = content_item__get_root_folder(null) 
-     or copy__folder_id = content_template__get_root_folder() 
-     or copy__target_folder_id = copy__folder_id then
-    v_valid_folders_p := 0;
+  if v_valid_folders_p != 2 then 
+    raise EXCEPTION ''-20000: content_folder.copy - Invalid folder(s)'';
   end if;
 
-    -- get the source folder info
-    select
-      name, label, description
-    into
-      v_name, v_label, v_description
-    from 
-      cr_items i, cr_folders f
-    where
-      f.folder_id = i.item_id
-    and
-      f.folder_id = copy__folder_id;
+  if copy__target_folder_id = copy__folder_id then 
+    raise EXCEPTION ''-20000: content_folder.copy - Cannot copy folder to itself'';
+  end if;
+  
+  if content_folder__is_sub_folder(copy__folder_id, copy__target_folder_id) = ''t'' then
+    raise EXCEPTION ''-20000: content_folder.copy - Destination folder is subfolder'';
+  end if;
 
-  if v_valid_folders_p = 2 then 
+  if content_folder__is_registered(copy__target_folder_id,''content_folder'',''f'') != ''t'' then
+    raise EXCEPTION ''-20000: content_folder.copy - Destination folder does not allow subfolders'';
+  end if;
 
-    if content_folder__is_sub_folder(copy__folder_id, copy__target_folder_id) != ''t'' or v_current_folder_id != copy__target_folder_id or (v_name != copy__name and copy__name is not null) then 
+  -- get the source folder info
+  select
+    name, label, description, parent_id
+  into
+    v_name, v_label, v_description, v_current_folder_id
+  from 
+    cr_items i, cr_folders f
+  where
+    f.folder_id = i.item_id
+  and
+    f.folder_id = copy__folder_id;
+
+  -- would be better to check if the copy__name alredy exists in the destination folder.
+
+  if v_current_folder_id = copy__target_folder_id and (v_name = copy__name or copy__name is null) then
+    raise EXCEPTION ''-20000: content_folder.copy - Destination folder is parent folder and folder alredy exists'';
+  end if;
 
       -- create the new folder
       v_new_folder_id := content_folder__new(
@@ -528,11 +602,13 @@ begin
 	  v_label,
 	  v_description,
 	  copy__target_folder_id,
-          null,
+	  copy__target_folder_id,
           null,
           now(),
 	  copy__creation_user,
-	  copy__creation_ip
+	  copy__creation_ip,
+          ''t'',
+          null
       );
 
       -- copy attributes of original folder
@@ -562,19 +638,18 @@ begin
 	    v_folder_contents_val.item_id,
 	    v_new_folder_id,
 	    copy__creation_user,
-	    copy__creation_ip    
+	    copy__creation_ip,
+            null
 	);
 
       end loop;
-    end if;
-  end if;
 
   return 0; 
 end;' language 'plpgsql';
 
 
 -- function is_folder
-select define_function_args('content_folder__is_folder','folder_id');
+select define_function_args('content_folder__is_folder','item_id');
 create or replace function content_folder__is_folder (integer)
 returns boolean as '
 declare
@@ -623,10 +698,10 @@ begin
     v_parent_id := v_rec.parent_id;
     exit when v_parent_id = is_sub_folder__folder_id;
     -- we did not find the folder, reset v_parent_id
-    v_parent_id := 0;
+    v_parent_id := -4;
   end LOOP;
 
-  if v_parent_id != 0 then 
+  if v_parent_id != -4 then 
     v_sub_folder_p := ''t'';
   end if;
 
@@ -895,7 +970,7 @@ declare
   v_is_root                       boolean;       
 begin
 
-  select parent_id = 0 into v_is_root 
+  select parent_id = -4 into v_is_root 
     from cr_items where item_id = is_root__folder_id;
 
   return v_is_root;
