@@ -4,7 +4,7 @@
 -- Copyright (C) 1999-2000 ArsDigita Corporation
 -- Author: Karl Goldstein (karlg@arsdigita.com)
 
--- $Id: content-revision.sql,v 1.35.2.2 2005/01/06 16:31:29 daveb Exp $
+-- $Id: content-revision.sql,v 1.46 2009/02/10 18:31:54 jeffd Exp $
 
 -- This is free software distributed under the terms of the GNU Public
 -- License.  Full text of the license is available from the GNU Project:
@@ -19,7 +19,7 @@
 -- than the standard package_instantiate_object.  So we don't bother calling define_function_args
 -- here.
 
-create or replace function content_revision__new (varchar,varchar,timestamptz,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar)
+create or replace function content_revision__new (varchar,varchar,timestamptz,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar,integer)
 returns integer as '
 declare
   new__title                  alias for $1;  
@@ -34,19 +34,30 @@ declare
   new__creation_date          alias for $9;  -- default now()
   new__creation_user          alias for $10; -- default null
   new__creation_ip            alias for $11; -- default null
+  new__package_id             alias for $12; -- default null  
   v_revision_id               integer;       
+  v_package_id                acs_objects.package_id%TYPE;
   v_content_type              acs_object_types.object_type%TYPE;
 begin
 
   v_content_type := content_item__get_content_type(new__item_id);
 
+  if new__package_id is null then
+    v_package_id := acs_object__package_id(new__item_id);
+  else
+    v_package_id := new__package_id;
+  end if;
+
   v_revision_id := acs_object__new(
       new__revision_id,
       v_content_type, 
       new__creation_date, 
       new__creation_user, 
       new__creation_ip, 
-      new__item_id
+      new__item_id,
+      ''t'',
+      new__title,
+      v_package_id
   );
 
   -- binary data is stored in cr_revisions using Dons lob hack.
@@ -71,6 +82,64 @@ begin
 
 end;' language 'plpgsql';
 
+create or replace function content_revision__new (varchar,varchar,timestamptz,varchar,varchar,integer,integer,integer,timestamptz,integer,varchar)
+returns integer as '
+declare
+  new__title                  alias for $1;  
+  new__description            alias for $2;  -- default null 
+  new__publish_date           alias for $3;  -- default now()
+  new__mime_type              alias for $4;  -- default ''text/plain''
+  new__nls_language           alias for $5;  -- default null
+  -- lob id 
+  new__data                   alias for $6;  
+  new__item_id                alias for $7;  
+  new__revision_id            alias for $8;  -- default null
+  new__creation_date          alias for $9;  -- default now()
+  new__creation_user          alias for $10; -- default null
+  new__creation_ip            alias for $11; -- default null
+begin
+        return content_revision__new(new__title,
+                                     new__description,
+                                     new__publish_date,
+                                     new__mime_type,
+                                     new__nls_language,
+                                     new__data,
+                                     new__item_id,
+                                     new__revision_id,
+                                     new__creation_date,
+                                     new__creation_user,
+                                     new__creation_ip,
+                                     null
+               );
+end;' language 'plpgsql';
+
+create or replace function content_revision__new(varchar,varchar,timestamptz,varchar,text,integer,integer) returns integer as '
+declare
+        new__title              alias for $1;
+        new__description        alias for $2;  -- default null
+        new__publish_date       alias for $3;  -- default now()
+        new__mime_type          alias for $4;  -- default ''text/plain''
+        new__text               alias for $5;  -- default '' ''
+        new__item_id            alias for $6;
+        new__package_id         alias for $7;  -- default null
+begin
+        return content_revision__new(new__title,
+                                     new__description,
+                                     new__publish_date,
+                                     new__mime_type,
+                                     null,
+                                     new__text,
+                                     new__item_id,
+                                     null,
+                                     now(),
+                                     null,
+                                     null,
+                                     null,
+                                     new__package_id
+               );
+
+end;' language 'plpgsql';
+
 create or replace function content_revision__new(varchar,varchar,timestamptz,varchar,text,integer) returns integer as '
 declare
         new__title              alias for $1;
@@ -90,6 +159,8 @@ begin
                                      null,
                                      now(),
                                      null,
+                                     null,
+                                     null,
                                      null
                );
 
@@ -121,11 +192,11 @@ begin
                                      new__creation_date,
                                      new__creation_user,
                                      new__creation_ip,
+                                     null,
                                      null
                                      );
 end;' language 'plpgsql';
 
--- function new
 create or replace function content_revision__new (varchar,varchar,timestamptz,varchar,varchar,text,integer,integer,timestamptz,integer,varchar,integer)
 returns integer as '
 declare
@@ -141,7 +212,42 @@ declare
   new__creation_user          alias for $10; -- default null
   new__creation_ip            alias for $11; -- default null
   new__content_length         alias for $12; -- default null
+begin
+        return content_revision__new(new__title,
+                                     new__description,
+                                     new__publish_date,
+                                     new__mime_type,
+                                     new__nls_language,
+                                     new__text,
+                                     new__item_id,
+                                     new__revision_id,
+                                     new__creation_date,
+                                     new__creation_user,
+                                     new__creation_ip,
+                                     new__content_length,
+                                     null
+                                     );
+end;' language 'plpgsql';
+
+-- function new
+create or replace function content_revision__new (varchar,varchar,timestamptz,varchar,varchar,text,integer,integer,timestamptz,integer,varchar,integer,integer)
+returns integer as '
+declare
+  new__title                  alias for $1;  
+  new__description            alias for $2;  -- default null  
+  new__publish_date           alias for $3;  -- default now()
+  new__mime_type              alias for $4;  -- default ''text/plain''
+  new__nls_language           alias for $5;  -- default null
+  new__text                   alias for $6;  -- default '' ''
+  new__item_id                alias for $7;  
+  new__revision_id            alias for $8;  -- default null
+  new__creation_date          alias for $9;  -- default now()
+  new__creation_user          alias for $10; -- default null
+  new__creation_ip            alias for $11; -- default null
+  new__content_length         alias for $12; -- default null
+  new__package_id             alias for $13; -- default null  
   v_revision_id               integer;       
+  v_package_id                acs_objects.package_id%TYPE;
   v_content_type              acs_object_types.object_type%TYPE;
   v_storage_type              cr_items.storage_type%TYPE;
   v_length                    cr_revisions.content_length%TYPE;
@@ -149,13 +255,22 @@ begin
 
   v_content_type := content_item__get_content_type(new__item_id);
 
+  if new__package_id is null then
+    v_package_id := acs_object__package_id(new__item_id);
+  else
+    v_package_id := new__package_id;
+  end if;
+
   v_revision_id := acs_object__new(
       new__revision_id,
       v_content_type, 
       new__creation_date, 
       new__creation_user, 
       new__creation_ip, 
-      new__item_id
+      new__item_id,
+      ''t'',
+      new__title,
+      v_package_id
   );
 
   select storage_type into v_storage_type
@@ -216,12 +331,13 @@ begin
     cols := cols || '', '' || attr_rec.attribute_name;
   end loop;
 
-  execute ''insert into '' || v_table_name || '' select '' || copy_attributes__copy_id || 
+    execute ''insert into '' || v_table_name || ''('' || v_id_column || cols || '')'' || '' select '' || copy_attributes__copy_id || 
           '' as '' || v_id_column || cols || '' from '' || 
           v_table_name || '' where '' || v_id_column || '' = '' || 
           copy_attributes__revision_id;
-  
+
   return 0; 
+
 end;' language 'plpgsql';
 
 
@@ -251,29 +367,43 @@ begin
   -- use the copy_id or generate a new copy_id if none is specified
   --   the copy_id is a revision_id
   if copy__copy_id is null then
-    select acs_object_id_seq.nextval into v_copy_id from dual;
+    select nextval(''t_acs_object_id_seq'') into v_copy_id from dual;
   else
     v_copy_id := copy__copy_id;
   end if;
 
   -- create the basic object
-  insert into acs_objects 
-       select 
-         v_copy_id as object_id, 
-         object_type, 
-         v_target_item_id, 
-         security_inherit_p, 
-         copy__creation_user as creation_user, 
-         now() as creation_date, 
+  insert into acs_objects (
+                 object_id,
+                 object_type,
+                 context_id,
+                 security_inherit_p,
+                 creation_user,
+                 creation_date,
+                 creation_ip,
+                 last_modified,
+                 modifying_user,
+                 modifying_ip,
+                 title,
+                 package_id)
+       select
+         v_copy_id as object_id,
+         object_type,
+         v_target_item_id,
+         security_inherit_p,
+         copy__creation_user as creation_user,
+         now() as creation_date,
          copy__creation_ip as creation_ip,
-         now() as last_modified, 
-         copy__creation_user as modifying_user, 
-         copy__creation_ip as modifying_ip 
+         now() as last_modified,
+         copy__creation_user as modifying_user,
+         copy__creation_ip as modifying_ip,
+         title,
+         package_id
        from
-         acs_objects 
-       where 
+         acs_objects
+       where
          object_id = copy__revision_id;
-  
+
   -- create the basic revision (using v_target_item_id)
   insert into cr_revisions 
       select 
@@ -292,23 +422,6 @@ begin
       where
         revision_id = copy__revision_id;
 
---                  select 
---                    object_type
---                  from                                                
---                    acs_object_types                                  
---                  where                                               
---                    object_type <> ''acs_object''                       
---                  and                                                 
---                    object_type <> ''content_revision''                 
---                  connect by                                          
---                    prior supertype = object_type                     
---                  start with                                          
-----                    object_type = (select object_type 
---                                     from acs_objects 
---                                    where object_id = copy__revision_id)
---                  order by
---                    level desc 
-
   -- iterate over the ancestor types and copy attributes
   for type_rec in select ot2.object_type, tree_level(ot2.tree_sortkey) as level
                   from acs_object_types ot1, acs_object_types ot2, acs_objects o
@@ -330,7 +443,7 @@ end;' language 'plpgsql';
 
 -- procedure delete
 select define_function_args('content_revision__del','revision_id');
-create or replace function content_revision__delete (integer)
+create or replace function content_revision__del (integer)
 returns integer as '
 declare
   delete__revision_id    alias for $1;  
@@ -403,7 +516,6 @@ begin
   return 0; 
 end;' language 'plpgsql';
 
-
 -- function get_number
 select define_function_args('content_revision__get_number','revision_id');
 create or replace function content_revision__get_number (integer)
@@ -449,52 +561,8 @@ begin
                from cr_revisions r where r.revision_id = p_revision_id;
 end;' language 'plpgsql' stable strict;
 
--- procedure index_attributes
-create or replace function content_revision__index_attributes (integer)
-returns integer as '
-declare
-  index_attributes__revision_id            alias for $1;  
-  clob_loc                                 text;          
-  v_revision_id                            cr_revisions.revision_id%TYPE;
-begin
-
-	raise exception ''content_revision__index_attributes not implemented on postgres'';
-
-end;' language 'plpgsql';
-
-
--- function import_xml
-create or replace function content_revision__import_xml (integer,integer,numeric)
-returns integer as '
-declare
-  import_xml__item_id                alias for $1;  
-  import_xml__revision_id            alias for $2;  
-  import_xml__doc_id                 alias for $3;  
-  clob_loc                           text;          
-  v_revision_id                      cr_revisions.revision_id%TYPE;
-begin
-
-	raise exception ''content_revision__import_xml not implemented on postgres'';
-
-end;' language 'plpgsql';
-
-
--- function export_xml
-create or replace function content_revision__export_xml (integer)
-returns integer as '
-declare
-  revision_id            alias for $1;  
-  clob_loc               text;          
-  v_doc_id               cr_xml_docs.doc_id%TYPE;
-  v_revision_id          cr_revisions.revision_id%TYPE;
-begin
-
-  raise exception ''content_revision__export_xml Not currently implemented on postgresql'';
- 
-end;' language 'plpgsql';
-
-
 -- procedure to_html
+select define_function_args('content_revision__to_html','revision_id');
 create or replace function content_revision__to_html (integer)
 returns integer as '
 declare
@@ -638,12 +706,16 @@ begin
     if v_storage_type = ''lob'' then
         v_new_lob := empty_lob();
 
+	PERFORM lob_copy(v_lob, v_new_lob);
+
         update cr_revisions
            set content = null,
                content_length = v_content_length,
                lob = v_new_lob
          where revision_id = v_revision_id_dest;
-        PERFORM lob_copy(v_lob, v_new_lob);
+	-- this call has to be before the above instruction,
+	-- because lob references the v_new_lob 
+	--        PERFORM lob_copy(v_lob, v_new_lob);
     else 
         -- this will work for both file and text types... well sort of.
         -- this really just creates a reference to the first file which is
