@@ -4,7 +4,7 @@
 -- Copyright (C) 1999-2000 ArsDigita Corporation
 -- Author: Karl Goldstein (karlg@arsdigita.com)
 
--- $Id: content-xml.sql,v 1.3 2001/04/28 19:58:39 donb Exp $
+-- $Id: content-xml.sql,v 1.4 2006/09/25 20:25:19 byronl Exp $
 
 -- This is free software distributed under the terms of the GNU Public
 -- License.  Full text of the license is available from the GNU Project:
@@ -21,7 +21,8 @@ create sequence cr_xml_doc_seq;
 -- ) on commit delete rows;
 
 create table cr_xml_docs (
-    doc_id        integer primary key,
+    doc_id        integer constraint cr_xml_docs_doc_id_pk 
+		  primary key,
     doc           text
 );
 
