@@ -10,7 +10,7 @@ ad_library {
     @creation-date 28 September 2000
     @author Henry Minsky (hqm@mit.edu)
     @author Lars Pind (lars@pinds.com)
-    @cvs-id $Id: locale-procs.tcl,v 1.27.2.2 2004/07/23 13:31:15 maltes Exp $
+    @cvs-id $Id: locale-procs.tcl,v 1.37 2010/01/08 17:39:58 emmar Exp $
 }
 
 namespace eval lang::system {}
@@ -50,14 +50,16 @@ ad_proc -public lang::system::site_wide_locale {
     return $parameter_locale
 }
 
-ad_proc -public lang::system::package_level_locale_not_cached {
+ad_proc -private lang::system::package_level_locale_not_cached {
     package_id
 } {
-    return [db_string get_system_locale {} -default {}]
+    return [db_string get_package_locale {} -default {}]
 }
 
 ad_proc -public lang::system::package_level_locale {
     package_id
+} {
+    @return empty string if not use_package_level_locales_p, or the package locale from apm_packages table.
 } {
     if { ![use_package_level_locales_p] } {
         return {}
@@ -79,7 +81,7 @@ ad_proc -public lang::system::locale {
         return [site_wide_locale]
     } 
 
-    if { [empty_string_p $package_id] && [ad_conn isconnected] } {
+    if { $package_id eq "" && [ad_conn isconnected] } {
         set package_id [ad_conn package_id]
     }
 
@@ -88,7 +90,7 @@ ad_proc -public lang::system::locale {
     set locale [package_level_locale $package_id]
 
     # If there's no package setting, use the site-wide setting
-    if { [empty_string_p $locale] } {
+    if { $locale eq "" } {
         set locale [site_wide_locale]
     } 
     return $locale
@@ -104,7 +106,7 @@ ad_proc -public lang::system::set_locale {
     @param package_id The package for which you want to set the locale setting, if you want to set system setting for one package only. Leave blank for site-wide setting.
     @param locale The new locale that you want to use as your system locale.
 } {
-    if { [empty_string_p $package_id] } {
+    if { $package_id eq "" } {
 
         parameter::set_value \
             -package_id [apm_package_id_from_key "acs-lang"] \
@@ -125,13 +127,25 @@ ad_proc -public lang::system::set_locale {
 ad_proc -public lang::system::language {
     {-package_id ""}
     {-site_wide:boolean}
+    {-iso6392:boolean}
 } {
     Get system language setting for a given package instance.
     
     @param package_id The package for which you want to get the language setting.
     @param site_wide Set this if you want to get the site-wide language setting.
+    @param iso6392   Set this if you want to force iso-639-2 code (3 digits)
+
+    @return 3 chars language code if iso6392 is set, left part of locale otherwise
+
 } {
-    return [string range [locale -package_id $package_id -site_wide=$site_wide_p] 0 1]
+    set locale [locale -package_id $package_id -site_wide=$site_wide_p]
+    set sys_lang [lindex [split $locale "_"] 0]
+
+    if { $iso6392_p } {
+        return [lang::util::iso6392_from_language -language $sys_lang]
+    } else {
+        return $sys_lang
+    }
 }
 
 ad_proc -public lang::system::timezone {} {
@@ -230,7 +244,7 @@ ad_proc -private lang::system::get_locales_not_cached {} {
     }]
 }
 
-ad_proc -public lang::system::get_locale_options_not_cached {} {
+ad_proc -private lang::system::get_locale_options_not_cached {} {
     Return all enabled locales in the system in a format suitable for the options argument of a form.
 
     @author Lars Pind
@@ -245,7 +259,7 @@ ad_proc -public lang::system::get_locale_options_not_cached {} {
 #
 #####
 
-ad_proc -public lang::user::package_level_locale_not_cached {
+ad_proc -private lang::user::package_level_locale_not_cached {
     user_id
     package_id
 } {
@@ -264,7 +278,7 @@ ad_proc -public lang::user::package_level_locale {
     given by its package id.
 } {
     # default to current user
-    if { [empty_string_p $user_id] } {
+    if { $user_id eq "" } {
         set user_id [ad_conn untrusted_user_id]
     }
 
@@ -283,33 +297,37 @@ ad_proc -public lang::user::site_wide_locale {
     Get the user's preferred site wide locale.
 } {
     # default to current user
-    if { [empty_string_p $user_id] } {
+    if { $user_id eq "" } {
         set user_id [ad_conn untrusted_user_id]
     }
 
     # For all the users with a user_id of 0 don't cache.
     if { $user_id == 0} {
-	return [lang::user::site_wide_locale_not_cached $user_id]
+    	return [lang::user::site_wide_locale_not_cached $user_id]
     }
 
     # Cache for the lifetime of sessions (7 days)
     return [util_memoize [list lang::user::site_wide_locale_not_cached $user_id] [sec_session_timeout]]
 }
 
-ad_proc -public lang::user::site_wide_locale_not_cached {
+ad_proc -private lang::user::site_wide_locale_not_cached {
     user_id
 } {
     Get the user's preferred site wide locale.
 } {
+    set system_locale [lang::system::site_wide_locale]
+
     if { $user_id == 0 } {
-	set locale [ad_get_cookie "ad_locale"]
-	if { [empty_string_p $locale] } {
-	    set locale [lang::system::site_wide_locale]
-	}
-	return $locale
+        set locale [ad_get_cookie "ad_locale"]
     } else {
-        return [db_string get_user_site_wide_locale {} -default ""]
+        set locale [db_string get_user_site_wide_locale {} -default "$system_locale"]
     }
+
+    if { $locale eq "" } {
+        set locale $system_locale
+    }
+
+    return $locale
 }
 
 ad_proc -public lang::user::locale {
@@ -324,12 +342,12 @@ ad_proc -public lang::user::locale {
     @param user_id Set this to the user you want to get the locale of, defaults to current user.
 } {
     # default to current user
-    if { [empty_string_p $user_id] } {
+    if { $user_id eq "" } {
         set user_id [ad_conn untrusted_user_id]
     }
 
     # default to current connection package
-    if { [empty_string_p $package_id] } {
+    if { $package_id eq "" } {
         set package_id [ad_conn package_id]
     }
 
@@ -337,7 +355,7 @@ ad_proc -public lang::user::locale {
     set locale [package_level_locale -user_id $user_id $package_id]
 
     # If there's no package setting, then use the site-wide setting
-    if { [empty_string_p $locale] } {
+    if { $locale eq "" } {
         set locale [site_wide_locale -user_id $user_id]
     } 
 
@@ -346,6 +364,7 @@ ad_proc -public lang::user::locale {
 
 ad_proc -public lang::user::set_locale {
     {-package_id ""}
+    {-user_id ""}
     locale
 } {
     Set user locale setting for a given package instance.
@@ -353,7 +372,9 @@ ad_proc -public lang::user::set_locale {
     @param package_id The package for which you want to set the locale setting, if you want to set it for a specific package, as opposed to a site-wide setting.
     @param locale The new locale that you want to use as your system locale.
 } {
-    set user_id [ad_conn user_id]
+    if { $user_id eq "" } {
+	set user_id [ad_conn user_id]
+    }
 
     if { $user_id == 0 } {
         # Not logged in, use a cookie-based client locale
@@ -364,7 +385,7 @@ ad_proc -public lang::user::set_locale {
         return
     }
 
-    if { [empty_string_p $package_id] } {
+    if { $package_id eq "" } {
         # Set site-wide locale in user_preferences table
         db_dml set_user_site_wide_locale {}
 
@@ -378,13 +399,13 @@ ad_proc -public lang::user::set_locale {
 
     set user_locale_exists_p [db_string user_locale_exists_p {}]
     if { $user_locale_exists_p } {
-        if { ![empty_string_p $locale] } {
+        if { $locale ne "" } {
             db_dml update_user_locale {}
         } else {
             db_dml delete_user_locale {}
         }
     } else {
-        if { ![empty_string_p $locale] } {
+        if { $locale ne "" } {
             db_dml insert_user_locale {}
         }
     }
@@ -396,14 +417,27 @@ ad_proc -public lang::user::set_locale {
 ad_proc -public lang::user::language {
     {-package_id ""}
     {-site_wide:boolean}
+    {-iso6392:boolean}
 } {
     Get user language preference for a given package instance.
     This preliminary implementation only has one site-wide setting, though.
     
     @param package_id The package for which you want to get the language setting.
     @param site_wide Set this if you want to get the site-wide language setting.
+    @param iso6392   Set this if you want to force iso-639-2 code (3 digits)
+
+    @return 3 chars language code if iso6392 is set, left part of locale otherwise
+
 } {
-    return [string range [locale -package_id $package_id -site_wide=$site_wide_p] 0 1]
+
+    set locale [locale -package_id $package_id -site_wide=$site_wide_p]
+    set user_lang [lindex [split $locale "_"] 0]
+
+    if { $iso6392_p } {
+        return [lang::util::iso6392_from_language -language $user_lang]
+    } else {
+        return $user_lang
+    }
 }
 
 
@@ -469,7 +503,7 @@ ad_proc -public lang::conn::locale {
 } {
     if { $site_wide_p } { 
         set locale [lang::user::site_wide_locale]
-        if { [empty_string_p $locale] } {
+        if { $locale eq "" } {
             set locale [lang::system::site_wide_locale]
         }
         return $locale
@@ -477,7 +511,7 @@ ad_proc -public lang::conn::locale {
 
     # default value for package_id
 
-    if { [empty_string_p $package_id] } {
+    if { $package_id eq "" } {
         set package_id [ad_conn package_id]
     }
 
@@ -487,32 +521,32 @@ ad_proc -public lang::conn::locale {
 
     # if that does not exist use system's package level locale
 
-    if { [empty_string_p $locale] } {
+    if { $locale eq "" } {
         set locale [lang::system::package_level_locale $package_id]
     } 
 
     # if that does not exist use user's site wide locale
 
-    if { [empty_string_p $locale] } {
+    if { $locale eq "" } {
         set locale [lang::user::site_wide_locale]
     } 
 
     # Use the accept-language browser heading
 
-    if { [empty_string_p $locale] } {
+    if { $locale eq "" } {
         set locale [lang::conn::browser_locale]
     }    
 
     # if that does not exist use system's site wide locale
 
-    if { [empty_string_p $locale] } {
+    if { $locale eq "" } {
         set locale [lang::system::site_wide_locale]
     } 
 
     # if that does not exist then we are back to just another language
     # let's pick uhmm... en_US
 
-    if { [empty_string_p $locale] } {
+    if { $locale eq "" } {
         set locale en_US
     } 
 
@@ -548,7 +582,7 @@ ad_proc -private lang::conn::browser_locale {} {
                 # i.e. a tentative match
                 if { ![info exists tentative_match] } {
                     set default_locale [lang::util::default_locale_from_lang $language]
-                    if { ![empty_string_p $default_locale] } {
+                    if { $default_locale ne "" } {
                         set tentative_match $default_locale
                     }                    
                 } else {
@@ -560,7 +594,7 @@ ad_proc -private lang::conn::browser_locale {} {
         } else {
             # We have just a language, e.g. en
             set default_locale [lang::util::default_locale_from_lang $locale]
-            if { ![empty_string_p $default_locale] } {
+            if { $default_locale ne "" } {
                 set perfect_match $default_locale
                 break
             }
@@ -605,13 +639,25 @@ ad_proc -private lang::conn::get_accept_language_header {} {
 ad_proc -public lang::conn::language {
     {-package_id ""}
     {-site_wide:boolean}
+    {-iso6392:boolean}
 } {
     Get the language for this request, perhaps for a given package instance.
     
     @param package_id The package for which you want to get the language.
     @param site_wide Set this if you want to get the site-wide language.
+    @param iso6392   Set this if you want to force the iso-639-2 code
+
+    @return 3 chars language code if iso6392 is set, left part of locale otherwise
 } {
-    return [string range [locale -package_id $package_id -site_wide=$site_wide_p] 0 1]
+
+    set locale [locale -package_id $package_id -site_wide=$site_wide_p]
+    set conn_lang [lindex [split $locale "_"] 0]
+
+    if { $iso6392_p } {
+        return [lang::util::iso6392_from_language -language $conn_lang]
+    } else {
+        return $conn_lang
+    }
 }
 
 ad_proc -public lang::conn::charset { 
@@ -640,7 +686,7 @@ ad_proc -public lang::conn::timezone {} {
         set timezone [lang::user::timezone]
     }
 
-    if { [empty_string_p $timezone] } {
+    if { $timezone eq "" } {
         # No user timezone, return the system timezone
         set timezone [lang::system::timezone]
     }
@@ -648,162 +694,24 @@ ad_proc -public lang::conn::timezone {} {
 }
 
 
-
 #####
 #
 # Backwards compatibility procs
 #
 #####
-
-ad_proc -deprecated -warn ad_locale {
-    context 
-    {item "locale"}
-} {
-    Returns the value of a locale item in a particular context. For example, to 
-    get the language, locale, and timezone preference for the current user:
-
-    <pre>
-    set user_lang [ad_locale user language] => en
-
-    set user_locale [ad_locale user locale] => en_US
-
-    set user_tz [ad_locale user timezone] => PST
-    </pre>
-
-    To get the preferred language of the current subsite:
-
-    <pre>
-    set user_lang [ad_locale subsite language] => ja
-    </pre>
-
-    Requires ad_locale_establish_vars to be run as a filter on each web page 
-    before this procedure is called.
-    @see           ad_locale_establish_vars
-    @author        Henry Minsky (hqm@mit.edu)
-    
-    @param context Context in which a locale value can be obtained.
-                   The only context that is currently implemented is user (for the current user).
-                   Examples of other contexts that could be implemented are:
-                   subsite (for the group that owns the current web page)
-    @param item    Specific item of data. 
-                   The only items that are implemented are locale, timezone and language.
-                   You can change the implementation to add other items as required.
-    @return        Value of the item in the specified context
-
-    @see lang::conn::locale
-    @see lang::user::locale
-    @see lang::user::language
-    @see lang::user::timezone
-    @see lang::util::charset_for_locale
-} {
-    switch $context {
-        request {
-	    switch $item {
-                locale {
-                    return [lang::conn::locale -site_wide]
-                }
-                language {
-                    return [lang::conn::language -site_wide]
-                }
-                timezone {
-                    return [lang::conn::timezone]
-                }
-                default {
-		    error "unsupported option to ad_locale: $item"
-                }
-            }
-        }
-	user {
-	    switch $item {
-		locale {
-		    return [lang::user::locale -site_wide]
-		}
-		language {
-		    return [lang::user::language -site_wide]
-		}
-		timezone {
-		    return [lang::user::timezone]
-		}
-		default {
-		    error "unsupported option to ad_locale: $item"
-		}
-	    }
-	}
-	charset {
-	    return [lang::util::charset_for_locale $item]
-	}
-	default {
-	    error "ad_locale: unknown context $context"
-	}
-    }
-}
-
-ad_proc -deprecated -warn ad_locale_set  { 
-    item 
-    value 
-} {
-    Sets the user's preferred locale info as a session var
-    <p>
-    usage:
-    <pre>
-    ad_locale_set locale "en_US"
-    ad_locale_set timezone "PST"
-    </pre>
-    @see lang::user::set_locale
-    @see lang::user::set_timezone
-} {
-    switch $item {
-        locale {
-            lang::user::set_locale $value
-        }
-        timezone {
-            lang::user::set_timezone $value
-        }
-        default {
-            error "Unknown item, $item"
-        }
-    }
-}
-
-ad_proc -deprecated -warn ad_locale_set_system_timezone { 
-    timezone
-}  { 
-    Tell OpenACS what timezone we think it's running in.
-    
-    @param timezone name from acs-reference package (e.g., Asia/Tokyo, America/New_York)
-    @see lang::system::set_timezone
-} {
-    lang::system::set_timezone $timezone
-}
-
-
-ad_proc -deprecated -warn ad_locale_get_system_timezone { }  { 
-    Ask OpenACS what it thinks our timezone is.
-
-    @return  a timezone name from acs-reference package (e.g., Asia/Tokyo, America/New_York)
-    @see lang::system::timezone
-} {
-    return [lang::system::timezone]
-}
-
-
-ad_proc -deprecated -warn ad_locale_system_tz_offset { } {
-    @return number of hours to subtract from local (Oracle) time to get UTC
-    @see lang::system::timezone_utc_offset
-} {
-    return [lang::system::timezone_utc_offset]
-}
-
-ad_proc -deprecated -public ad_locale_get_label { locale } {
+ad_proc -deprecated -warn -public ad_locale_get_label { locale } {
 
     Returns the label (name) of locale
 
+    To be removed in 5.3
+
     @author	Bruno Mattarollo (bruno.mattarollo@ams.greenpeace.org)
 
     @param locale	Code for the locale, eg "en_US"
 
     @return	String containing the label for the locale
 
+    @see lang::util::get_label
 } {
     return [db_string select_locale_label {
         select label 
