@@ -4,7 +4,7 @@ ad_page_contract {
 
     @author Jon Griffin (jon@jongriffin.com)
     @creation-date 2001-08-26
-    @cvs-id $Id: index.tcl,v 1.2 2004/02/05 23:34:31 jong Exp $
+    @cvs-id $Id: index.tcl,v 1.3 2005/03/01 00:01:22 jeffd Exp $
 } {
 } -properties {
   context_bar:onevalue
@@ -16,7 +16,7 @@ ad_page_contract {
 set title "Reference Data"
 set package_id [ad_conn package_id]
 set context_bar [list $title]
-set user_id [ad_verify_and_get_user_id]
+set user_id [ad_conn user_id]
 
 set admin_p [ad_permission_p $package_id admin]
 
