@@ -4,7 +4,7 @@ ad_page_contract {
     @param repository_id 
     @author Jon Griffin (jon@jongriffin.com)
     @creation-date 17 Sept 2001
-    @cvs-id $Id: view-one-reference.tcl,v 1.2 2004/02/05 23:36:08 jong Exp $
+    @cvs-id $Id: view-one-reference.tcl,v 1.3 2005/03/01 00:01:22 jeffd Exp $
 } {
     repository_id:integer,notnull
 } -properties {
@@ -17,7 +17,7 @@ ad_page_contract {
 set package_id [ad_conn package_id]
 set title "View one Table"
 set context_bar [list [list "reference-list" "Reference List" ] "$title"]
-set user_id [ad_verify_and_get_user_id]
+set user_id [ad_conn user_id]
 
 db_1row get_table { *SQL* } -column_array table_info
 
