@@ -4,16 +4,23 @@
 --
 -- @creation-date 28 August 2000
 --
--- @cvs-id $Id: acs-subsite-create.sql,v 1.3.6.1 2004/11/23 20:01:11 vivianh Exp $
+-- @cvs-id $Id: acs-subsite-create.sql,v 1.8 2009/12/08 01:57:22 donb Exp $
 --
 
-@@ attribute
+-- create table email_image_rel_ext (
+--	rel_id 		integer constraint email_image_rel_ext_fk references acs_rels(rel_id)
+--			constraint email_image_rel_ext primary key 
+-- );
+
+
 @@ portraits
+@@ email-image
 @@ application-groups-create
 @@ subsite-callbacks-create
 @@ host-node-map-create
 @@ user-sc-create
-@@ site-node-delection
+@@ site-node-selection
+@@ themes-create
 
 -- This view lets us avoid using acs_object.name to get party_names.
 -- 
@@ -30,3 +37,4 @@ from parties p,
      persons
 where p.party_id = groups.group_id(+)
   and p.party_id = persons.person_id(+);
+
