@@ -6,7 +6,7 @@ ad_page_contract {
 
     @author mbryzek@arsdigita.com
     @creation-date Tue Nov  7 12:14:42 2000
-    @cvs-id $Id: enum-add.tcl,v 1.2 2002/09/06 21:49:57 jeffd Exp $
+    @cvs-id $Id: enum-add.tcl,v 1.3 2007/01/10 21:22:06 gustafn Exp $
 
 } {
     attribute_id:integer,notnull
@@ -40,7 +40,7 @@ set max_values 5
 template::multirow create value_form sort_order field_name
 
 for { set i 1 } { $i <= $max_values } { incr i } {
-    template::multirow append value_form [expr $i + $number_values] "attribute_enum_values.[expr $i + $number_values]"
+    template::multirow append value_form [expr {$i + $number_values}] "attribute_enum_values.[expr {$i + $number_values}]"
 }
 
 db_1row select_attr_name {
