@@ -6,7 +6,7 @@ ad_page_contract {
 
     @author mbryzek@arsdigita.com
     @creation-date Sun Dec 10 14:48:44 2000
-    @cvs-id $Id: value-delete-2.tcl,v 1.1.1.1 2001/03/13 22:59:26 ben Exp $
+    @cvs-id $Id: value-delete-2.tcl,v 1.2 2007/01/10 21:22:06 gustafn Exp $
 
 } {
     attribute_id:naturalnum,notnull
@@ -15,7 +15,7 @@ ad_page_contract {
     { return_url "one?[ad_export_vars attribute_id]" }    
 }
 
-if { [string eq $operation "Yes, I really want to delete this attribute value"] } {
+if {$operation eq "Yes, I really want to delete this attribute value"} {
     db_transaction {
 	attribute::value_delete $attribute_id $enum_value
     }
