@@ -6,7 +6,7 @@ ad_page_contract {
     @author Oumi Mehrotra (oumi@arsdigita.com)
 
     @creation-date 2001-02-23
-    @cvs-id $Id: change-join-policy.tcl,v 1.3 2002/09/06 21:49:58 jeffd Exp $
+    @cvs-id $Id: change-join-policy.tcl,v 1.4 2007/01/10 21:22:06 gustafn Exp $
 } {
     group_type:notnull
     {return_url ""}
@@ -37,7 +37,7 @@ if { ![db_0or1row select_pretty_name {
     return
 }
 
-if {![string equal $dynamic_p t]} {
+if {$dynamic_p ne "t" } {
     ad_return_error "Cannot administer group type" "Group type \"$group_type\" can only be administered by programmers"
 }
 
