@@ -6,7 +6,7 @@ ad_page_contract {
     @author Oumi Mehrotra (oumi@arsdigita.com)
 
     @creation-date 2001-02-23
-    @cvs-id $Id: change-join-policy-2.tcl,v 1.1.1.1 2001/03/13 22:59:26 ben Exp $
+    @cvs-id $Id: change-join-policy-2.tcl,v 1.2 2007/01/10 21:22:07 gustafn Exp $
 } {
     group_id:integer,notnull
     join_policy:notnull
@@ -32,7 +32,7 @@ db_dml update_join_policy {
     where group_id = :group_id
 }
 
-if {[empty_string_p $return_url]} {
+if {$return_url eq ""} {
     set return_url one?group_id=@group_id@
 }
 
