@@ -5,7 +5,7 @@ ad_page_contract {
     @author Michael Bryzek (mbryzek@arsdigita.com)
 
     @creation-date August 15, 2000
-    @cvs-id $Id: index.tcl,v 1.13 2004/03/02 14:55:55 lars Exp $
+    @cvs-id $Id: index.tcl,v 1.19 2011/04/15 09:50:52 emmar Exp $
 } {
 } -properties {
     context:onevalue
@@ -17,9 +17,11 @@ ad_page_contract {
 }
 
 array set this_node [site_node::get -url [ad_conn url]]
-set subsite_name $this_node(instance_name)
+set title "$this_node(instance_name) [_ acs-subsite.Administration]"
 
 set acs_admin_url [apm_package_url_from_key "acs-admin"]
 array set acs_admin_node [site_node::get -url $acs_admin_url]
 set acs_admin_name $acs_admin_node(instance_name)
 set sw_admin_p [permission::permission_p -party_id [ad_conn user_id] -object_id $acs_admin_node(object_id) -privilege admin]
+
+set convert_subsite_p [expr { [llength [apm::get_package_descendent_options [ad_conn package_key]]] > 0 }]
