@@ -1,6 +1,6 @@
 ad_page_contract {
     Interface for specifying a list of users to sign up as a batch
-    @cvs-id $Id: user-batch-add.tcl,v 1.1.2.1 2004/03/19 13:09:35 peterm Exp $
+    @cvs-id $Id: user-batch-add.tcl,v 1.4 2009/01/06 02:25:34 gustafn Exp $
 } -properties {
     context:onevalue
     system_name:onevalue
@@ -11,7 +11,7 @@ ad_page_contract {
 
 subsite::assert_user_may_add_member
 
-set admin_user_id [ad_verify_and_get_user_id]
+set admin_user_id [ad_conn user_id]
 set admin_email [db_string unused "select email from 
 parties where party_id = :admin_user_id"]
 set administration_name [db_string admin_name "select
@@ -20,6 +20,6 @@ first_names || ' ' || last_name from persons where person_id = :admin_user_id"]
 set context [list [list "./" "Users"] "Notify added user"]
 set system_name [ad_system_name]
 set export_vars [export_form_vars email first_names last_name user_id]
-set system_url [ad_parameter -package_id [ad_acs_kernel_id] SystemURL ""].
+set system_url [parameter::get -package_id [ad_acs_kernel_id] -parameter SystemURL -default ""].
 
 ad_return_template
