@@ -1,13 +1,13 @@
 ad_page_contract {
 
-    @cvs-id $Id: unsubscribe.tcl,v 1.5 2003/09/25 16:31:24 lars Exp $
+    @cvs-id $Id: unsubscribe.tcl,v 1.6 2009/03/31 14:23:12 emmar Exp $
 }
 
 set user_id [auth::get_user_id -account_status closed]
 
 set system_name [ad_system_name]
 
-set page_title "Confirm Closing Your Account"
+set page_title [_ acs-subsite.Close_your_account]
 set context [list [list [ad_pvt_home] [ad_pvt_home_name]] $page_title]
 
 set pvt_home [ad_pvt_home]
