@@ -506,12 +506,11 @@ ad_proc -public ad_form {
     @see ad_set_form_values
 
 } {
-
     set level [template::adp_level]
 
     # Are we extending the form?
 
-    if { [string equal [lindex $args 0] "-extend"] } {
+    if {[lindex $args 0] eq "-extend"} {
         set extend_p 1
         set args [lrange $args 1 end]
     } else {
@@ -527,8 +526,8 @@ ad_proc -public ad_form {
     set valid_args { form method action mode html name select_query select_query_name new_data
                      on_refresh edit_data validate on_submit after_submit confirm_template
                      on_request new_request edit_request export cancel_url cancel_label
-                     has_submit has_edit actions edit_buttons display_buttons show_required_p
-                     on_validation_error };
+                     has_submit has_edit actions edit_buttons display_buttons show_required_p 
+                     on_validation_error fieldset };
 
     ad_arg_parser $valid_args $args
 
@@ -547,12 +546,16 @@ ad_proc -public ad_form {
         return -code error "Can't extend form \"$form_name\" - a parameter block requiring the full form has already been declared"
     }
 
-    global af_parts
+    # Allow an empty form to work until we see an action block, useful for building up
+    # forms piecemeal.
 
-    if { $extend_p && ![info exists af_parts(${form_name}__form)] } {
-        return -code error "You can't extend form \"$form_name\" until you've created the form"
+    global af_element_names
+    if { !$extend_p } {
+        set af_element_names($form_name) [list]
     }
 
+    global af_parts
+
     foreach valid_arg $valid_args {
         if { [info exists $valid_arg] } {
             if { [info exists af_parts(${form_name}__$valid_arg)] &&
@@ -566,21 +569,12 @@ ad_proc -public ad_form {
             # and validation block to be extended, for now at least until I get more experience
             # with this ...
 
-            if { [lsearch { name form method action html validate export mode cancel_url has_edit has_submit actions edit_buttons display_buttons on_validation_error} $valid_arg ] == -1 } {
+            if { [lsearch { name form method action html validate export mode cancel_url has_edit has_submit actions edit_buttons display_buttons fieldset on_validation_error} $valid_arg ] == -1 } {
                 set af_parts(${form_name}__extend) ""
             }
         }
     }
 
-    if { ![info exists af_parts(${form_name}__form)] } {
-        return -code error "No \"form\" block has been specified for form \"$form_name\""
-    }
-
-    # If we're not extending - this needs integration with the ATS form builder ...
-    if { !$extend_p } {
-        # incr ad_conn(form_count)
-    }
-
     ####################
     #
     # Step 1: Parse the form specification
@@ -592,7 +586,6 @@ ad_proc -public ad_form {
     # and we should extend its data and use it directly, but there's not time to do this
     # right for Greenpeace so I'm hacking the hell out of it)
 
-    global af_element_names
     global af_flag_list
     global af_to_sql
     global af_from_sql
@@ -605,18 +598,19 @@ ad_proc -public ad_form {
     array set af_element_parameters [list] 
 
     if { [info exists form] } {
-
+	
         # Remove comment lines in form section (DanW)
         regsub -all -line -- {^\s*\#.*$} $form "" form
-
+	
         foreach element $form {
             set element_name_part [lindex $element 0]
 
             # This can easily be generalized if we add more embeddable form commands ...
 
-            if { [string equal $element_name_part "-section"] } {
-                lappend af_element_names($form_name) "[list "-section" [uplevel [list subst [lindex $element 1]]]]"
+            if {$element_name_part eq "-section"} {
+                lappend af_element_names($form_name) "[concat "-section" [uplevel [list subst [lrange $element 1 end]]]]"
             } else {
+                set element_name_part [uplevel [list subst $element_name_part]]
                 if { ![regexp {^([^ \t:]+)(?::([a-zA-Z0-9_,(|)]*))?$} $element_name_part match element_name flags] } {
                     return -code error "Form element '$element_name_part' doesn't have the right format. It must be var\[:flag\[,flag ...\]\]"
                 }
@@ -631,11 +625,11 @@ ad_proc -public ad_form {
                     set af_element_parameters($element_name:$flag) [list]
                     set left_paren [string first "(" $flag]
                     if { $left_paren != -1 } {
-                        if { ![string equal [string index $flag end] ")"] } {
+                        if { [string index $flag end] ne ")" } {
                             return -code error "Missing or misplaced end parenthesis for flag '$flag' on argument '$element_name'"
                         }
-                        set flag_stem [string range $flag 0 [expr $left_paren - 1]]
-                        lappend af_element_parameters($element_name:$flag_stem) [string range $flag [expr $left_paren + 1] [expr [string length $flag]-2]]
+                        set flag_stem [string range $flag 0 [expr {$left_paren - 1}]]
+                        lappend af_element_parameters($element_name:$flag_stem) [string range $flag [expr {$left_paren + 1}] [expr {[string length $flag]-2}]]
                         lappend af_flag_list(${form_name}__$element_name) $flag_stem
                     } else {
                         lappend af_flag_list(${form_name}__$element_name) $flag
@@ -651,7 +645,9 @@ ad_proc -public ad_form {
     # checks.  We implement this by building a global list of validation elements
 
     global af_validate_elements
-    set af_validate_elements($form_name) [list]
+    if { !$extend_p } {
+        set af_validate_elements($form_name) [list]
+    }
 
     if { [info exists validate] } {
 
@@ -663,7 +659,8 @@ ad_proc -public ad_form {
                 return -code error "Validate block must have three arguments: element name, expression, error message"
             }
 
-            if { [lsearch $af_element_names($form_name) [lindex $validate_element 0]] == -1 } {
+            if {[lsearch $af_element_names($form_name) [lindex $validate_element 0]] == -1
+                && ![template::element::exists $form_name [lindex $validate_element 0]]} { 
                 return -code error "Element \"[lindex $validate_element 0]\" is not a form element"
             }
             lappend af_validate_elements($form_name) $validate_element
@@ -717,6 +714,10 @@ ad_proc -public ad_form {
             lappend create_command "-display_buttons" $display_buttons
         }
 
+        if { [info exists fieldset] } {
+            lappend create_command "-fieldset" $fieldset
+        }
+
         if { [info exists show_required_p] } {
             lappend create_command "-show_required_p" $show_required_p
         }
@@ -740,6 +741,9 @@ ad_proc -public ad_form {
 
         template::element create $form_name __refreshing_p -datatype integer -widget hidden -value 0
 
+	# add the hidden button element
+	template::element create $form_name "__submit_button_name" -datatype text -widget hidden -value ""
+	template::element create $form_name "__submit_button_value" -datatype text -widget hidden -value ""
     }
 
     if { [info exists export] } {
@@ -762,10 +766,24 @@ ad_proc -public ad_form {
     global af_sequence_name
 
     foreach element_name $element_names {
-        if { [llength $element_name] == 2 } {
-            switch [string range [lindex $element_name 0] 1 end] {
-                section { template::form section $form_name [lindex $element_name 1] }
+        if { [lindex $element_name 0] eq "-section" } {
+            set command [list template::form section]
+            foreach {option} [lrange $element_name 2 end] {
+                set switch [lindex $option 0]
+                set args [lindex $option 1]
+                switch $switch {
+                    fieldset -
+                    legendtext -
+                    legend {
+                        lappend command -$switch
+                        lappend command $args
+                    }
+                    default {return -code error "\"$switch\" is not a legal -section option"}
+                }
             }
+            lappend command $form_name
+            lappend command [lindex $element_name 1]
+            eval $command
         } else {
             set form_command [list template::element create $form_name $element_name]
             foreach flag $af_flag_list(${form_name}__$element_name) {
@@ -776,7 +794,8 @@ ad_proc -public ad_form {
                             return -code error "element $element_name: a form can only declare one key"
                         }
                         set af_key_name($form_name) $element_name
-                        if { ![empty_string_p $af_element_parameters($element_name:key)] } {
+                        set af_type(${form_name}__$element_name) integer
+                        if { $af_element_parameters($element_name:key) ne "" } {
                             if { [info exists af_sequence_name($form_name)] } {
                                 return -code error "element $element_name: duplicate sequence"
                             }
@@ -784,18 +803,19 @@ ad_proc -public ad_form {
                         }
                         lappend form_command "-datatype" "integer" "-widget" "hidden"
                         template::element create $form_name __key_signature -datatype text -widget hidden -value ""
+                        template::element create $form_name __key -datatype text -widget hidden -value $element_name
                         template::element create $form_name __new_p -datatype integer -widget hidden -value 0
                     }
 
                     multiple {
-                        if { ![empty_string_p $af_element_parameters($element_name:$flag)] } {
+                        if { $af_element_parameters($element_name:$flag) ne "" } {
                             return -code error "element $element_name: $flag attribute can not have a parameter"
                         }
                     }
 
                     nospell -
 		    optional {
-                        if { ![empty_string_p $af_element_parameters($element_name:$flag)] } {
+                        if { $af_element_parameters($element_name:$flag) ne "" } {
                             return -code error "element $element_name: $flag attribute can not have a parameter"
                         }
                         lappend form_command "-$flag"
@@ -804,15 +824,15 @@ ad_proc -public ad_form {
                     from_sql -
                     to_sql -
                     to_html {
-                        if { [empty_string_p $af_element_parameters($element_name:$flag)] } {
+                        if { $af_element_parameters($element_name:$flag) eq "" } {
                             return -code error "element $element_name: \"$flag\" attribute must have a parameter"
                         }
                         set name af_$flag
+                        global af_$flag
                         append name "(${form_name}__$element_name)"
                         if { [info exists $name] } {
                             return -code error "element $element_name: \"$flag\" appears twice"
                         }
-                        global $name
                         set $name $af_element_parameters($element_name:$flag)
                     }
 
@@ -823,7 +843,7 @@ ad_proc -public ad_form {
                         lappend form_command "-datatype"
                         lappend form_command $flag
                         set af_type(${form_name}__$element_name) $flag
-                        if { [empty_string_p $af_element_parameters($element_name:$flag)] } {
+                        if { $af_element_parameters($element_name:$flag) eq "" } {
                             if { ![empty_string_p [info command "::template::widget::$flag"]] } {
                                 lappend form_command "-widget" $flag
                             }
@@ -848,21 +868,20 @@ ad_proc -public ad_form {
 
     # Check that any acquire and get_property attributes are supported by their element's datatype
     # These are needed at submission and fill-the-form-with-db-values time 
-
     foreach element_name $af_element_names($form_name) {
         if { [llength $element_name] == 1 } {
             if { [info exists af_from_sql(${form_name}__$element_name)] } {
-                if { [empty_string_p [info commands "::template::util::$af_type(${form_name}__$element_name)::acquire"]] } {
+                if { [info commands "::template::util::$af_type(${form_name}__$element_name)::acquire"] eq "" } {
                     return -code error "\"from_sql\" not valid for type \"$af_type(${form_name}__$element_name)\""
                 }
             }
             if { [info exists af_to_sql(${form_name}__$element_name)] } {
-                if { [empty_string_p [info commands "::template::util::$af_type(${form_name}__$element_name)::get_property"]] } {
+                if { [info commands ::template::util::$af_type(${form_name}__$element_name)::get_property] eq "" } {
                     return -code error "\"to_sql\" not valid for type \"$af_type(${form_name}__$element_name)\""
                 }
             }
             if { [info exists af_to_html(${form_name}__$element_name)] } {
-                if { [empty_string_p [info commands "::template::util::$af_type(${form_name}__$element_name)::get_property"]] } {
+                if { [empty_string_p [info commands ::template::util::$af_type(${form_name}__$element_name)::get_property]] } {
                     return -code error "\"to_html\" not valid for type \"$af_type(${form_name}__$element_name)\""
                 }
             }
@@ -886,6 +905,10 @@ ad_proc -public ad_form {
         return
     }
 
+    if { ![info exists af_parts(${form_name}__form)] } {
+        return -code error "No \"form\" block has been specified for form \"$form_name\""
+    }
+
     if { [template::form is_request $form_name] } {
 
         upvar #$level __ad_form_values__ values
@@ -948,6 +971,8 @@ ad_proc -public ad_form {
                             if { [info exists af_from_sql(${form_name}__$element_name)] } {
                                 set values($element_name) [template::util::$af_type(${form_name}__$element_name)::acquire \
                                                            $af_from_sql(${form_name}__$element_name) $values($element_name)]
+                            } elseif { [info commands ::template::data::from_sql::$af_type(${form_name}__$element_name)] ne "" } {
+                                set values($element_name) [template::data::from_sql::$af_type(${form_name}__$element_name) $values($element_name)]
                             }
                         }
                     }
@@ -972,7 +997,7 @@ ad_proc -public ad_form {
                     return -code error "Couldn't get the next value from sequence: $errmsg\""
                 }
                 set values(__new_p) 1
-
+		
                 if { [info exists new_request] } {
                     ad_page_contract_eval uplevel #$level $new_request
                     # LARS: Set form values based on local vars in the new_request block
@@ -1006,6 +1031,7 @@ ad_proc -public ad_form {
 
         # Get all the form elements.  We can't call form get_values because it doesn't handle multiples
         # in a reasonable way.
+
         foreach element_name $properties(element_names) {
             if { [info exists af_flag_list(${form_name}__$element_name)] && \
                  [lsearch $af_flag_list(${form_name}__$element_name) multiple] >= 0 } {
@@ -1017,6 +1043,13 @@ ad_proc -public ad_form {
             }
         }
 
+	# Update the clicked button if it does not already exist
+	uplevel #$level {
+		if {![exists_and_not_null ${__submit_button_name}]} {
+		    set ${__submit_button_name} ${__submit_button_value}
+		}
+	    }
+
         if { [info exists key_name] } {
             upvar #$level $key_name __key
             upvar #$level __key_signature __key_signature
@@ -1069,7 +1102,6 @@ ad_proc -public ad_form {
                             lappend args [list $element_name [uplevel #$level [list set $element_name]]]
                         }
                     }
-
                     # This is serious abuse of ad_return_exception_template, but hell, I wrote it so I'm entitled ...
                     ad_return_exception_template -status 200 -params $args $confirm_template
 
@@ -1199,7 +1231,8 @@ ad_proc -public ad_set_form_values {
             upvar $arg value
             ad_set_element_value -element $arg -- $value
         } else {
-            ad_set_element_value -element [lindex $arg 0] -- [lindex $arg 1]
+            set value [uplevel subst \{[lindex $arg 1]\}]
+            ad_set_element_value -element [lindex $arg 0] -- $value
         }
     }
 }
@@ -1239,6 +1272,6 @@ ad_proc -public ad_form_new_p {
 
     set form [ns_getform]
 
-    return [expr {[empty_string_p $form] || [ns_set find $form $key] == -1 || [ns_set get $form __new_p] == 1 }]
+    return [expr {$form eq "" || [ns_set find $form $key] == -1 || [ns_set get $form __new_p] == 1 }]
 
 }
