@@ -2,7 +2,7 @@
 
 # Copyright (C) 1999-2000 ArsDigita Corporation
 # Author: Karl Goldstein (karlg@arsdigita.com)
-# $Id: template-init.tcl,v 1.2 2002/03/09 02:00:02 donb Exp $
+# $Id: template-init.tcl,v 1.3 2008/12/21 17:28:52 gustafn Exp $
 
 # This is free software distributed under the terms of the GNU Public
 # License.  Full text of the license is available from the GNU Project:
@@ -12,11 +12,11 @@
 
 set pkg_id [apm_package_id_from_key acs-templating]
 
-if { [ad_parameter -package_id $pkg_id ShowCompiledTemplatesP dummy 0] } {
+if { [parameter::get -package_id $pkg_id -parameter ShowCompiledTemplatesP -default 0] } {
   ad_register_filter postauth GET *.cmp cmp_page_filter
 }
 
-if { [ad_parameter -package_id $pkg_id ShowDataDictionariesP  dummy 0] } {
+if { [parameter::get -package_id $pkg_id -parameter ShowDataDictionariesP -default 0] } {
   ad_register_filter postauth GET *.dat dat_page_filter
   ad_register_filter postauth GET *.frm frm_page_filter
 }
